package algod;

/**
 * Message transmis entre deux sommets.
 * @author <A href=http://www.alapetite.fr.st>Alexandre Alapetite</A>, <A href=http://www.andujar.fr.st>Brice Andujar</A>, <A href=mailto:gregory.gontier@free.fr>Gregory Gontier</A>
 * @version 1.0
 * @see AlgoDMessageFIFO
 * @see Sommet#recevoir
 * @see SommetVoisin#recevoir
 */
public class AlgoDMessage extends Object
{
      //------------Election----------------------------------------------------
  /**
   * Transmission de l'indentifiant.
   */
  public static final char MSG_ELECT_IDENTIFIANT = 'I';

  /**
   * Accus de rception positif.
   */
  public static final char MSG_ELECT_OUI = 'O';

  /**
   * Accus de rception ngatif.
   * //@deprecated les messages ngatifs ne sont plus envoys.
   */
  //public static final char MSG_ELECT_NON = 'N';

  /**
   * Diffusion de l'identifiant de l'lu.
   */
  public static final char MSG_ELECT_ELU = 'E';

      //-----------Arbre couvrant-----------------------------------------------
  /**
   * Diffusion de l'indentifiant d'une proposition de parent.
   */
  public static final char MSG_ARBCOUV_PARENT = 'P';

  /**
   * Transmission de l'identifiant d'un enfant  son pre.
   */
  public static final char MSG_ARBCOUV_ENFANT_OUI = 'F';

  /**
   * Transmission de l'identifiant d'un enfant  son pre.
   */
  public static final char MSG_ARBCOUV_ENFANT_NON = 'G';

  /**
   * Transmission de la fin de l'algorithme d'arbre couvrant.
   */
  public static final char MSG_ARBCOUV_FINI = 'C';

      //-----------Numrotation en profondeur-----------------------------------
  /**
   * Nouvelle numrotation en profondeur suite  l'arbre couvrant.
   */
  public static final char MSG_NUMPROF_IDPROF = 'D';

  /**
   * Transmission du numro du fils par le pre
   */
  public static final char MSG_NUMPROF_SETIDPROF = 'V';

      //-----------Coloration---------------------------------------------------
  /**
   * Transmission de sa couleur  ses voisins plus faibles.
   */
  public static final char MSG_COLOR_NUMCOLOR = 'K';

  /**
   * Transmission de sa couleur  ses fils et demande de coloration.
   */
  public static final char MSG_COLOR_DEMARRE = 'U';

  /**
   * Transmission  son pre d'une coloration russie.
   */
  public static final char MSG_COLOR_OK = 'Y';

  /**
   * Transmission  son pre d'une coloration ratte.
   */
  public static final char MSG_COLOR_NON = 'Q';

      //----------Coloration Asynchrone 1---------------------------------------
  /**
   * Transmission de sa couleur  ses voisins plus faibles.
   */
  public static final char MSG_COLORASYNCH1_NUMCOLOR = 'k';

  /**
   * Transmission de sa couleur  ses voisins plus faibles sans jeton.
   */
  public static final char MSG_COLORASYNCH1_NUMCOLOR_SJ = 'h';

  /**
   * Transmission de sa couleur  ses fils et demande de coloration.
   */
  public static final char MSG_COLORASYNCH1_DEMARRE = 'u';

  /**
   * Transmission de sa couleur  ses fils et demande de coloration.
   */
  public static final char MSG_COLORASYNCH1_DEMARRE_SJ = 'w';

  /**
   * Transmission  son pre d'une coloration russie.
   */
  public static final char MSG_COLORASYNCH1_OK = 'y';

  /**
   * Transmission  son pre d'une coloration russie sans jeton.
   */
  public static final char MSG_COLORASYNCH1_OK_SJ = '1';

  /**
   * Transmission  son pre d'une coloration ratte.
   */
  public static final char MSG_COLORASYNCH1_NON = 'q';

  /**
   * Transmission  son pre d'une coloration ratte sans jeton.
   */
  public static final char MSG_COLORASYNCH1_NON_SJ = '2';

      //------------------------------------------------------------------------
  /**
   * Type de message transmis.
   */
  public final char typeMessage;

  /**
   * Contenu du message, dpendant du type.
   */
  public final String contenu;

  /**
   * Sommet qui est la cause de ce message.
   */
  public final int identSommetSource;

  /**
   * Dernier sommet (voisin) ayant relay ce message.
   */
  public final int identVoisin;

  /**
   * Dans le cas d'un message rout, identifiant du sommet  qui
   * ce message est destin.
   */
  public final int identSommetDestination;

  public AlgoDMessage(char typeMessage, String contenu, int identSommetSource, int identVoisin, int identSommetDestination)
  {
    super();
    this.typeMessage = typeMessage;
    this.contenu = contenu;
    this.identSommetSource = identSommetSource;
    this.identVoisin = identVoisin;
    this.identSommetDestination = identSommetDestination;
  }

  /**
   * Informations sur ce message.
   * @return un texte sur une ligne dcrivant ce message.
   */
  public String toString()
  {
    return ""+typeMessage+"{"+contenu+"}"+" de "+identSommetSource+" par "+identVoisin+" pour "+identSommetDestination;
  }

}
