package algod;

/**
 * Vecteur de boolens  agrandissement automatique.
 * @author <A href=http://www.alapetite.fr.st>Alexandre Alapetite</A>, <A href=http://www.andujar.fr.st>Brice Andujar</A>, <A href=mailto:gregory.gontier@free.fr>Gregory Gontier</A>
 * @version 1.0
 */
public class BooleanVector extends Object
{
  /**
   * Tableau des boolens.
   */
  private boolean[] table;

  /**
   * Valeur par dfaut lors de la cration de nouvelles cases.
   */
  private final boolean defautValue;

  /**
   * Cre un nouveau tableau de boolean.
   * @param initialCapacity la taille initiale du tableau.
   * @param defautValue valeur avec laquelle les nouvelles cellules sont initialises.
   */
  public BooleanVector(int initialCapacity, boolean defautValue)
  {
    super();
    this.defautValue = defautValue;
    this.table = new boolean[initialCapacity];
    set(0,initialCapacity-1,defautValue);
  }

  /**
   * Cre un nouveau tableau de boolean. <BR>
   * Les nouvelles cellules sont initialises avec <CODE>false</CODE>.
   * @param initialCapacity la taille initiale du tableau.
   */
  public BooleanVector(int initialCapacity)
  {
    this(initialCapacity,false);
  }

  /**
   * Cre un nouveau tableau de boolean. <BR>
   * La taille initiale du tableau est de 10.<BR>
   * Les nouvelles cellules sont initialises avec <CODE>false</CODE>.
   */
  public BooleanVector()
  {
    this(10);
  }

  /**
   * Destructeur du tableau. Appel facultatif.
   */
  public void destroy()
  {
    table = null;
  }

  /**
   * Remplit une portion du vecteur avec value.
   */
  public final void set(int fromIndex, int toIndex, boolean value)
  {
    for (int cpt=fromIndex; cpt<=toIndex; cpt++)
      table[cpt] = value;
  }

  /**
   * Rinitialise tout le tableau avec la valeur par dfaut.
   */
  public final void clear()
  {
    set(0,table.length-1,defautValue);
  }

  /**
   * Change la taille du tableau. <BR>
   * Recopie les anciennes cellules et initialise les nouvelles avec <CODE>defautValue</CODE>.
   */
  final void setSize(int newSize)
  {
    boolean[] temp = table;
    table = new boolean[newSize];
    System.arraycopy(temp, 0, table, 0, Math.min(temp.length,newSize));
    set(temp.length,newSize-1,defautValue);
  }

  /**
   * Change la taille du tableau si ncessaire. <BR>
   * Agrandissement de 50%.
   * @see #setSize
   */
  final void setSizeIf(int minSize)
  {
    if (minSize>table.length) setSize(3 * minSize / 2);
  }

  /**
   * Accs  un lment du tableau. <BR>
   * Le tableau est agrandit si ncessaire.
   * @return la valeur du boolen  cet index.
   */
  public final boolean get(int index)
  {
    setSizeIf(index+1);
    return table[index];
  }

  /**
   * Change la valeur d'un lment du tableau. <BR>
   * Le tableau est agrandit si ncessaire.
   * @param index index de l'lment modifi.
   * @param value valeur de l'lment.
   */
  public final void set(int index, boolean value)
  {
    setSizeIf(index+1);
    table[index] = value;
  }

  /**
   * Met  <CODE>true</CODE> un lment du tableau. <BR>
   * Le tableau est agrandit si ncessaire.
   * @param index index de l'lment modifi.
   */
  public final void set(int index)
  {
    set(index,true);
  }

  /**
   * Met  <CODE>false</CODE> un lment du tableau. <BR>
   * Le tableau est agrandit si ncessaire.
   * @param index index de l'lment modifi.
   */
  public final void clear(int index)
  {
    set(index,false);
  }

  /**
   * Fait un et-logique sur un lment du tableau. <BR>
   * Le tableau est agrandit si ncessaire.
   * @param index index de l'lment modifi.
   * @param value valeur avec laquelle le et-logique est effectu.
   */
  public final boolean and(int index, boolean value)
  {
    setSizeIf(index+1);
    table[index] &= value;
    return table[index];
  }

  /**
   * Fait un ou-logique sur un lment du tableau. <BR>
   * Le tableau est agrandit si ncessaire.
   * @param index index de l'lment modifi.
   * @param value valeur avec laquelle le ou-logique est effectu.
   */
  public final boolean or(int index, boolean value)
  {
    table[index] |= value;
    return table[index];
  }

  /**
   * @return la taille du tableau.
   */
  public final int capacity()
  {
    return table.length;
  }

}
