package algod;

/**
 * Vecteur d'entiers  agrandissement automatique.
 * @author <A href=http://www.alapetite.fr.st>Alexandre Alapetite</A>, <A href=http://www.andujar.fr.st>Brice Andujar</A>, <A href=mailto:gregory.gontier@free.fr>Gregory Gontier</A>
 * @version 1.0
 */
public class IntVector extends Object
{
  /**
   * Tableau des entiers.
   */
  private int[] table;

  /**
   * Valeur par dfaut lors de la cration de nouvelles cases.
   */
  private final int defautValue;

  /**
   * Cre un nouveau tableau de int.
   * @param initialCapacity la taille initiale du tableau.
   * @param defautValue valeur avec laquelle les nouvelles cellules sont initialises.
   */
  public IntVector(int initialCapacity, int defautValue)
  {
    super();
    this.defautValue = defautValue;
    this.table = new int[initialCapacity];
    set(0,initialCapacity-1,defautValue);
  }

  /**
   * Cre un nouveau tableau de int. <BR>
   * Les nouvelles cellules sont initialises avec des 0.
   * @param initialCapacity la taille initiale du tableau.
   */
  public IntVector(int initialCapacity)
  {
    this(initialCapacity,0);
  }

  /**
   * Cre un nouveau tableau de int. <BR>
   * La taille initiale du tableau est de 10.<BR>
   * Les nouvelles cellules sont initialises avec des 0.
   */
  public IntVector()
  {
    this(10);
  }

  /**
   * Destructeur du tableau. Appel facultatif.
   */
  public void destroy()
  {
    table = null;
  }

  /**
   * Remplit une portion du vecteur avec value.
   */
  public final void set(int fromIndex, int toIndex, int value)
  {
    for (int cpt=fromIndex; cpt<=toIndex; cpt++)
      table[cpt] = value;
  }

  /**
   * Rinitialise tout le tableau avec la valeur par dfaut.
   */
  public final void clear()
  {
    set(0,table.length-1,defautValue);
  }

  /**
   * Change la taille du tableau. <BR>
   * Recopie les anciennes cellules et initialise les nouvelles avec <CODE>defautValue</CODE>.
   */
  final void setSize(int newSize)
  {
    int[] temp = table;
    table = new int[newSize];
    System.arraycopy(temp, 0, table, 0, Math.min(temp.length,newSize));
    set(temp.length,newSize-1,defautValue);
  }

  /**
   * Change la taille du tableau si ncessaire. <BR>
   * Agrandissement de 50%.
   * @see #setSize
   */
  final void setSizeIf(int minSize)
  {
    if (minSize>table.length) setSize(3 * minSize / 2);
  }

  /**
   * Accs  un lment du tableau. <BR>
   * Le tableau est agrandit si ncessaire.
   * @return la valeur de l'entier  cet index.
   */
  public final int get(int index)
  {
    setSizeIf(index+1);
    return table[index];
  }

  /**
   * Indique si la valeur de l'lment  cet index est diffrente de la valeur par dfaut.
   * Le tableau est agrandit si ncessaire.
   * @return true si la valeur  cet index est diffrente de <CODE>defautValue</CODE>, false sinon.
   * @see #defautValue
   */
  public final boolean isSet(int index)
  {
    setSizeIf(index+1);
    return table[index]!=defautValue;
  }

  /**
   * Change la valeur d'un lment du tableau. <BR>
   * Le tableau est agrandit si ncessaire.
   * @param index index de l'lment modifi.
   * @param value valeur de l'lment.
   */
  public final void set(int index, int value)
  {
    setSizeIf(index+1);
    table[index] = value;
  }

  /**
   * Incrmente de 1 la valeur d'un lment. <BR>
   * Le tableau est agrandit si ncessaire.
   * @param index index de l'lment modifi.
   */
  public final int inc(int index)
  {
    setSizeIf(index+1);
    return ++table[index];
  }

  /**
   * Dcrmente de 1 la valeur d'un lment. <BR>
   * Le tableau est agrandit si ncessaire.
   * @param index index de l'lment modifi.
   */
  public final int dec(int index)
  {
    setSizeIf(index+1);
    return --table[index];
  }

  /**
   * @return la taille du tableau.
   */
  public final int capacity()
  {
    return table.length;
  }

}
