package algod;

import graphelastic.SommetInterface;

/**
 * Dfini pour un sommet les mthodes propres aux threads.
 * Sommet compltement dfini.
 * @author <A href=http://www.alapetite.fr.st>Alexandre Alapetite</A>, <A href=http://www.andujar.fr.st>Brice Andujar</A>, <A href=mailto:gregory.gontier@free.fr>Gregory Gontier</A>
 * @version 1.0
 */
public final class Sommet extends SommetColorationAsynch1 implements SommetInterface
{
  /**
   * Indique si le thread est en cours d'excution
   */
  boolean tourne;

  /**
   * Contrleur  prvenir lors de la destruction effective de ce Sommet(Thread).
   */
  private GrapheInterface arretNotify;

  public Sommet(int couleur, int identifiant)
  {
    super(couleur,identifiant);
    this.tourne = true;
    this.arretNotify = null;
  }

  /**
   * Dtruit le thread sans pravis.
   * @see #arrete
   */
  public void destroy()
  {
    this.tourne = false;
    super.destroy();
    if (arretNotify != null) arretNotify.desinscription(this);
    arretNotify = null;
  }

  /**
   * Arrte l'excution du thread proprement. <BR>
   * Il s'auto-dtruira ensuite par l'appel de <CODE>destroy()</CODE>
   * @see #destroy()
   */
  public void arrete(GrapheInterface arretNotify)
  {
    this.arretNotify = arretNotify;
    this.tourne = false;
    this.yield(); //c'est le processus appelant qui fait une pause, pas le thread de ce sommet !
  }

  /**
   * Appelle les mthodes de dmarrage parentes, puis boucle tant que <CODE>tourne</CODE>
   * est vrai, en traitant les messages ventuellements en attente.
   * @see #tourne
   * @see #algoDMessageFIFO
   */
  public void run()
  {
    super.run();
    while (tourne)
      if (isFini()) this.yield();
      else traiteReception(algoDMessageFIFO.pop());
    destroy();
  }

  /**
   * Texte  afficher pour ce sommet.
   */
  public String getLabel()
  {
    return Integer.toString(identifiant)+","+Integer.toString(getIdentifiantProfondeur());
  }

  protected void traiteReception(AlgoDMessage adm)
  {
    //if (verbose) affiche("<-"+identifiant+" recoit "+adm.toString());
    int numVoisin = numOfVoisin(adm.identVoisin);
    //if (numVoisin<0) afficheErreur("erreur de numero voisin");
    //else
    super.traiteReception(adm,numVoisin);
  }

  public String toString()
  {
    return super.toString()+"; tourne="+tourne;
  }

}
