package algod;

/**
 * Dfini la coloration pour un sommet.
 * @author <A href=http://www.alapetite.fr.st>Alexandre Alapetite</A>, <A href=http://www.andujar.fr.st>Brice Andujar</A>, <A href=mailto:gregory.gontier@free.fr>Gregory Gontier</A>
 * @version 1.0
 */
class SommetColoration extends SommetNumProfondeur
{
  /**
   * Compteur de back-tracks.
   */
  static int nbBackTrack = 0;

  /**
   * Compteur de messages pour la coloration.
   */
  static int nbMessagesColoration = 0;

  /**
   * Utilise la mthode de backtrack optimise : remonte au plus petit voisin plus grand que soi.
   */
  static boolean backtrack_optimise = true;

  /**
   * Couleur de ce sommet pour les problmes de coloration.
   */
  int couleur;

  /**
   * Couleur maximum pour ce sommet.
   */
  protected final int couleurMax;

  /**
   * Table des couleurs utilises par les voisins plus forts.
   */
  BooleanVector booleanVector;

  /**
   * Voisin qui route en cas de back-track d'erreur.
   */
  private int numVoisinBackTrack;

  /**
   * Indentifiant du sommet
   */
  private int identVoisinBackTrack;

  /**
   * Voisin qui route en cas de back-track aller.
   */
  private int numVoisinTrack;

  private int numEnfantTrack;

  /**
   * Voisin qui route en cas de back-track russi et fini.
   */
  private int numVoisinTrackFini;

  /**
   * Indique si le routage pour le back-track a dj t recherch.
   */
  boolean voisinsTrackConnus;


  //private int compteurFilsColore;

  /**
   * @param identifiant identifiant unique de ce sommet.
   * @param couleurMax couleur de ce sommet pour les problmes de coloration. Initialis  la couleur maximum souhaite.
   */
  public SommetColoration(int couleurMax, int identifiant)
  {
    super(identifiant);
    this.couleurMax = couleurMax;
    this.couleur = couleurMax;
    this.voisinsTrackConnus = false;
    this.booleanVector = new BooleanVector(couleurMax+1,false);
  }

  public void destroy()
  {
    booleanVector.destroy();
    booleanVector = null;
    super.destroy();
  }

  /**
   * Accs  la couleur de ce sommet.
   */
  public final int getCouleur()
  {
    return couleur;
  }

  /**
   * Envoie un message aux voisins plus forts.
   */
  public final void envoyerAuxForts(char typeMessage, String contenu, int identSommetSource)
  {
    SommetVoisin vSupp;
    for (int cpt=nbVoisins()-1; cpt>=0; cpt--)
    {
      vSupp = getVoisinAt(cpt);
      if (vSupp.getIdentifiantProfondeur() > this.getIdentifiantProfondeur())
        envoyerASommet(typeMessage,contenu,identSommetSource,vSupp);
    }
  }

  /**
   * Envoie un message aux voisins plus faibles.
   */
  public final void envoyerAuxFaibles(char typeMessage, String contenu, int identSommetSource)
  {
    SommetVoisin vSupp;
    for (int cpt=nbVoisins()-1; cpt>=0; cpt--)
    {
      vSupp = getVoisinAt(cpt);
      if (vSupp.getIdentifiantProfondeur() < this.getIdentifiantProfondeur())
        envoyerASommet(typeMessage,contenu,identSommetSource,vSupp);
    }
  }

  protected void demarreColoration()
  {
    SommetIO.rndDelaiTransmission = 500;
    affiche("~--- "+identifiant+" lance la coloration en "+couleurMax+" ------");
    this.couleur = this.couleurMax;
    envoyerAuxFaibles(AlgoDMessage.MSG_COLOR_NUMCOLOR,Integer.toString(couleur),identifiant);
    chercheRoutesBackTrack();
    envoyerATrack(AlgoDMessage.MSG_COLOR_DEMARRE,"",this.getIdentifiantProfondeur());
  }

  /**
   * Mthode invoque lorsque l'algorithme de coloration est termin. <BR>
   * Ce sommet est de couleur couleurMax.
   * @see #couleurMax
   */
  protected void finColoration(boolean ok)
  {
    if (ok) affiche("~--- Coloration en "+couleurMax+" finie pour "+identifiant+" ("+nbBackTrack+" backtrack)---");
    else affiche("~--- Coloration en "+couleurMax+" impossible pour "+identifiant+" ("+nbBackTrack+" backtrack)---");
  }

  /**
   * Termine la mthode hrite de numrotation en profondeur pour dmarrer la coloration.
   */
  protected final void finNumProfondeur()
  {
    super.finNumProfondeur();
    System.gc(); //appelle un coup le garbage collector
    demarreColoration();
  }

  int getCouleurMaxPossible()
  {
    int uneCouleur = this.couleurMax;
    while (booleanVector.get(uneCouleur) && (uneCouleur>0)) uneCouleur--;
    return uneCouleur;
  }

  /**
   * Dsactive les couleurs utilises parmis les voisins d'identifiant suprieur.
   */
  void setCouleursVoisinsSupp()
  {
    SommetVoisin vSupp;
    for (int cpt=nbVoisins()-1; cpt>=0; cpt--)
    { //parcours les sommets d'identifiant suprieur
      vSupp = getVoisinAt(cpt);
      if (vSupp.getIdentifiantProfondeur() > this.getIdentifiantProfondeur())
        booleanVector.set(vSupp.getCouleur());
    }
  }

  /**
   * Recherche les routes pour le back-track.
   */
  void chercheRoutesBackTrack()
  {
    this.numVoisinTrackFini = this.numVoisinParent;
    numVoisinTrack = -1;
    int maxVoisinsInf = -1;
    numVoisinBackTrack = -1;
    numEnfantTrack = -1;
    for (int cpt=nbEnfants()-1; cpt>=0; cpt--) // corriger
    {
      int idProf = getEnfantAt(cpt).getIdentifiantProfondeur();
      if (idProf == this.getIdentifiantProfondeur()+1)
        numVoisinBackTrack = numOfVoisin(getEnfantAt(cpt).getIdentifiant());
      else if (idProf == this.getIdentifiantProfondeur()-1)
      {
        numEnfantTrack = cpt;
        numVoisinTrack = numOfVoisin(getEnfantAt(numEnfantTrack).getIdentifiant());
      }
    }
    if (numVoisinTrack<0) numVoisinTrack = this.numVoisinParent;
    if (numVoisinBackTrack<0) numVoisinBackTrack = numVoisinParent;
    identVoisinBackTrack = this.getIdentifiantProfondeur()+1;
    this.voisinsTrackConnus = true;

    //optimis
    int mini = Integer.MAX_VALUE;
    for (int cpt=nbVoisins()-1; cpt>=0; cpt--)
    {
      int idProf = getVoisinAt(cpt).getIdentifiantProfondeur();
      if ((idProf<mini) && (idProf > this.getIdentifiantProfondeur()))
      {
        mini = idProf;
        numVoisinBackTrack = cpt;
      }
    }
    //numVoisinBackTrack = this.numVoisinParent;
    if (numVoisinBackTrack>=0)
      identVoisinBackTrack = getVoisinAt(numVoisinBackTrack).getIdentifiantProfondeur();
    if (identVoisinBackTrack == identElu) //cas de la racine comme prdcesseur
    {
      identVoisinBackTrack = this.getIdentifiantProfondeur()+1;
      numVoisinBackTrack = numVoisinParent;
    }
  }

  /**
   * Envoie un message au sommet prcdent dans le backtrack.
   */
  final void envoyerABackTrack(char typeMessage, String contenu, int identSommetSource)
  {
    envoyerAVoisin(typeMessage,contenu,identSommetSource,numVoisinBackTrack,identVoisinBackTrack);
  }

  final void envoyerAEnfantsPasTrack(char typeMessage, String contenu, int identSommetSource)
  {
    for (int cpt=nbEnfants()-1; cpt>=0; cpt--)
    {
      if (cpt!=numEnfantTrack)
        envoyerASommet(typeMessage,contenu,identSommetSource,getEnfantAt(cpt));
    }
  }

  /**
   * Envoie un message au sommet suivant dans le backtrack.
   */
  final void envoyerATrack(char typeMessage, String contenu, int identSommetSource)
  {
    envoyerAVoisin(typeMessage,contenu,identSommetSource,numVoisinTrack,this.getIdentifiantProfondeur()-1);
  }

  /**
   * Envoie un message de fin de back-track russi.
   */
  final void envoyerABackTrackFini(char typeMessage, String contenu, int identSommetSource)
  {
    envoyerAVoisin(typeMessage,contenu,identSommetSource,numVoisinTrackFini,this.identElu);
  }

  protected void traiteReception(AlgoDMessage adm, int numVoisin)
  {
    switch (adm.typeMessage)
    {
      case AlgoDMessage.MSG_COLOR_NUMCOLOR  :
        nbMessagesColoration++;
        receptionColoration(numVoisin,Integer.parseInt(adm.contenu));
        break;
      case AlgoDMessage.MSG_COLOR_OK :
        nbMessagesColoration++;
        receptionOK(adm.identSommetSource);
        break;
      case AlgoDMessage.MSG_COLOR_NON :
        nbMessagesColoration++;
        receptionNON(adm.identSommetSource,adm.identSommetDestination);
        break;
      case AlgoDMessage.MSG_COLOR_DEMARRE :
        nbMessagesColoration++;
        receptionDemarre(adm.identSommetSource,adm.identSommetDestination);
        break;
      default : super.traiteReception(adm, numVoisin);
    }
  }

  /**
   * Un voisin nous transmet sa nouvelle couleur.
   */
  private void receptionColoration(int numVoisin, int colore)
  {
    getVoisinAt(numVoisin).setCouleur(colore);
  }

  /**
   * Le back-track s'est termin et il est russi.
   */
  private void receptionOK(int identSource)
  {
    if (isElu()) finColoration(true);
    else envoyerABackTrackFini(AlgoDMessage.MSG_COLOR_OK,"",identSource); //routage
  }

  private void receptionNON(int identSource, int identDest)
  {
    if (identDest == this.getIdentifiantProfondeur()) //le message nous est adress
    {
      couleur = getCouleurMaxPossible();
      booleanVector.set(couleur);
      if (couleur<=0) colorationRattee();
      else
      {
        envoyerAuxFaibles(AlgoDMessage.MSG_COLOR_NUMCOLOR,Integer.toString(couleur),identifiant);
        envoyerATrack(AlgoDMessage.MSG_COLOR_DEMARRE,"",this.getIdentifiantProfondeur());
      }
    }
    else //routage
    {
      int numVoisBT = -1;
      int minVoisinsSup = Integer.MAX_VALUE;
      if (identDest>this.getIdentifiantProfondeur()) numVoisBT = numVoisinParent;
      else for (int cpt=nbEnfants()-1; cpt>=0; cpt--)
      {
        int idProf = getEnfantAt(cpt).getIdentifiantProfondeur();
        if ((idProf<minVoisinsSup) && (idProf>=identDest))
        { //Minimum des fils suprieur ou gal  la destination
          numVoisBT = numOfVoisin(getEnfantAt(cpt).getIdentifiant());
          minVoisinsSup = idProf;
        }
      }
      envoyerAVoisin(AlgoDMessage.MSG_COLOR_NON,"",identSource,numVoisBT,identDest);
    }
  }

  /**
   * Un voisin (normalement notre pre dans l'arbre couvrant), nous transmet
   * sa couleur et nous demande de nous colorer.
   */
  private void receptionDemarre(int identSource, int identDest)
  {
    if (identDest == this.getIdentifiantProfondeur())
    {
      if (!voisinsTrackConnus) chercheRoutesBackTrack(); //si on n'a pas encore cherch les chemins de routage pour nous
      booleanVector.clear();
      setCouleursVoisinsSupp();
      couleur = getCouleurMaxPossible();
      booleanVector.set(couleur);
      if (couleur<=0) colorationRattee();
      else if (this.getIdentifiantProfondeur()<=0) receptionOK(identifiant); //backtrack termin et russi
      else
      {
        envoyerAuxFaibles(AlgoDMessage.MSG_COLOR_NUMCOLOR,Integer.toString(couleur),identifiant);
        envoyerATrack(AlgoDMessage.MSG_COLOR_DEMARRE,"",this.getIdentifiantProfondeur());
      }
    }
    else //routage
    {
      int numVoisTrack = -1;
      int maxVoisinsInf = -1;
      if (getIdentifiantProfondeur()-poids>identDest) numVoisTrack = numVoisinParent;
      else for (int cpt=nbEnfants()-1; cpt>=0; cpt--)
      {
        int idProf = getEnfantAt(cpt).getIdentifiantProfondeur();
        if ((idProf>maxVoisinsInf) && (idProf<=identDest))
        {
          numVoisTrack = numOfVoisin(getEnfantAt(cpt).getIdentifiant());
          maxVoisinsInf = idProf; //Minimum des fils infrieur ou gal  la destination
        }
      }
      if (numVoisTrack<0) numVoisTrack = this.numVoisinParent;
      envoyerAVoisin(AlgoDMessage.MSG_COLOR_DEMARRE,"",identSource,numVoisTrack,identDest);
    }
  }

  /**
   * On ne peut pas se colorer, on back-traque.
   */
  private void colorationRattee()
  {
    if (verbose) affiche("--"+identifiant+"("+getIdentifiantProfondeur()+") backtrack---------");
    nbBackTrack++;
    if (isElu()) finColoration(false);
    else envoyerABackTrack(AlgoDMessage.MSG_COLOR_NON,"",identifiant);
  }

  public String toString()
  {
    return super.toString()+"; couleur="+couleur;
  }

}
