package algod;

/**
 * Coloration avec paralllisme (colorations secondaires).
 * @author <A href=http://www.alapetite.fr.st>Alexandre Alapetite</A>, <A href=http://www.andujar.fr.st>Brice Andujar</A>, <A href=mailto:gregory.gontier@free.fr>Gregory Gontier</A>
 * @version 1.0
 */
public class SommetColorationAsynch1 extends SommetColoration
{
  /**
   * Utilisation de la coloration parallle secondaire.
   */
  static boolean colorationAsynch1 = true;

  /**
   * Nombre de back-tracks secondaires.
   */
  static int nbBackTrack_SJ = 0;

  /**
   * Nombre de messages pour la coloration secondaire.
   */
  static int nbMessagesColoration_SJ = 0;

  /**
   * Indique si le sommet a une couleur officielle.
   */
  private boolean coloreParJeton;

  public SommetColorationAsynch1(int couleurMax, int identifiant)
  {
    super(couleurMax,identifiant);
    coloreParJeton = false;
    this.couleur = 0;
  }

  public final boolean isCouleurOfficielle()
  {
    return coloreParJeton;
  }

  protected void demarreColoration()
  {
    SommetIO.rndDelaiTransmission = 1000;
    if (colorationAsynch1)
    {
      affiche("~--- "+identifiant+" lance la coloration asynchrone1 en "+couleurMax+" ------");
      this.couleur = this.couleurMax;
      coloreParJeton = true;
      envoyerAuxFaibles(AlgoDMessage.MSG_COLORASYNCH1_NUMCOLOR,Integer.toString(couleur),identifiant);
      chercheRoutesBackTrack();
      envoyerATrack(AlgoDMessage.MSG_COLORASYNCH1_DEMARRE,"",this.getIdentifiantProfondeur());
      envoyerAEnfantsPasTrack(AlgoDMessage.MSG_COLORASYNCH1_DEMARRE_SJ,Integer.toString(couleur),identifiant);
    }
    else super.demarreColoration();
  }

  protected void traiteReception(AlgoDMessage adm, int numVoisin)
  {
    switch (adm.typeMessage)
    {
      case AlgoDMessage.MSG_COLORASYNCH1_NUMCOLOR :
        nbMessagesColoration++;
        receptionColoration(numVoisin,Integer.parseInt(adm.contenu));
        break;
      case AlgoDMessage.MSG_COLORASYNCH1_NUMCOLOR_SJ :
        nbMessagesColoration_SJ++;
        receptionColoration_SJ(numVoisin,Integer.parseInt(adm.contenu));
        break;
      case AlgoDMessage.MSG_COLORASYNCH1_OK :
        nbMessagesColoration++;
        receptionOK(adm.identSommetSource);
        break;
      case AlgoDMessage.MSG_COLORASYNCH1_OK_SJ :
        nbMessagesColoration_SJ++;
        receptionOK_SJ(adm.identSommetSource);
        break;
      case AlgoDMessage.MSG_COLORASYNCH1_NON :
        nbMessagesColoration++;
        receptionNON(adm.identSommetSource,adm.identSommetDestination);
        break;
      case AlgoDMessage.MSG_COLORASYNCH1_NON_SJ :
        nbMessagesColoration_SJ++;
        receptionNON_SJ(adm.identSommetSource,adm.identSommetDestination);
        break;
      case AlgoDMessage.MSG_COLORASYNCH1_DEMARRE :
        nbMessagesColoration++;
        receptionDemarre(adm.identSommetSource,adm.identSommetDestination);
        break;
      case AlgoDMessage.MSG_COLORASYNCH1_DEMARRE_SJ :
        nbMessagesColoration_SJ++;
        receptionDemarre_SJ(adm.identSommetSource,Integer.parseInt(adm.contenu),adm.identSommetDestination);
        break;
      default : super.traiteReception(adm, numVoisin);
    }
  }

  /**
   * Un voisin nous transmet sa nouvelle couleur.
   */
  private void receptionColoration(int numVoisin, int colore)
  {
    getVoisinAt(numVoisin).setCouleur(colore);
  }

  private void receptionColoration_SJ(int numVoisin, int colore)
  {
    getVoisinAt(numVoisin).setCouleur(colore);
  }

  /**
   * Le back-track s'est termin et il est russi.
   */
  private void receptionOK(int identSource)
  {
    if (isElu()) finColoration(true);
    else envoyerABackTrackFini(AlgoDMessage.MSG_COLORASYNCH1_OK,"",identSource); //routage
  }

  private void receptionOK_SJ(int identSource)
  {
    if (!isElu()) envoyerABackTrackFini(AlgoDMessage.MSG_COLORASYNCH1_OK_SJ,"",identSource); //routage
  }

  private void receptionNON(int identSource, int identDest)
  {
    if (identDest == this.getIdentifiantProfondeur()) //le message nous est adress
    {
      couleur = getCouleurMaxPossible();
      booleanVector.set(couleur);
      if (couleur<=0) colorationRattee();
      else
      {
        envoyerAuxFaibles(AlgoDMessage.MSG_COLORASYNCH1_NUMCOLOR,Integer.toString(couleur),identifiant);
        envoyerATrack(AlgoDMessage.MSG_COLORASYNCH1_DEMARRE,Integer.toString(couleur),this.getIdentifiantProfondeur());
        //envoyerAEnfantsPasTrack(AlgoDMessage.MSG_COLORASYNCH1_DEMARRE_SJ,Integer.toString(couleur),this.getIdentifiantProfondeur());
      }
    }
    else //routage
    {
      int numVoisBT = -1;
      int minVoisinsSup = Integer.MAX_VALUE;
      if (identDest>this.getIdentifiantProfondeur()) numVoisBT = numVoisinParent;
      else for (int cpt=nbEnfants()-1; cpt>=0; cpt--)
      {
        int idProf = getEnfantAt(cpt).getIdentifiantProfondeur();
        if ((idProf<minVoisinsSup) && (idProf>=identDest))
        { //Minimum des fils suprieur ou gal  la destination
          numVoisBT = numOfVoisin(getEnfantAt(cpt).getIdentifiant());
          minVoisinsSup = idProf;
        }
      }
      envoyerAVoisin(AlgoDMessage.MSG_COLORASYNCH1_NON,"",identSource,numVoisBT,identDest);
    }
  }

  private void receptionNON_SJ(int identSource, int identDest)
  {
    if (identDest == this.getIdentifiantProfondeur()) //le message nous est adress
    {
      if (!coloreParJeton) //on n'est pas color par une couleur officielle
      {
        couleur = getCouleurMaxPossible();
        booleanVector.set(couleur);
        if (couleur<=0) colorationRattee_SJ();
        else
        {
          envoyerAuxFaibles(AlgoDMessage.MSG_COLORASYNCH1_NUMCOLOR_SJ,Integer.toString(couleur),identifiant);
          //envoyerATrack(AlgoDMessage.MSG_COLORASYNCH1_DEMARRE_SJ,Integer.toString(couleur),this.getIdentifiantProfondeur());
        }
      }
    }
    else //routage
    {
      int numVoisBT = -1;
      int minVoisinsSup = Integer.MAX_VALUE;
      if (identDest>this.getIdentifiantProfondeur()) numVoisBT = numVoisinParent;
      else for (int cpt=nbEnfants()-1; cpt>=0; cpt--)
      {
        int idProf = getEnfantAt(cpt).getIdentifiantProfondeur();
        if ((idProf<minVoisinsSup) && (idProf>=identDest))
        { //Minimum des fils suprieur ou gal  la destination
          numVoisBT = numOfVoisin(getEnfantAt(cpt).getIdentifiant());
          minVoisinsSup = idProf;
        }
      }
      envoyerAVoisin(AlgoDMessage.MSG_COLORASYNCH1_NON_SJ,"",identSource,numVoisBT,identDest);
    }
  }

  /**
   * Un voisin (normalement notre pre dans l'arbre couvrant), nous transmet
   * sa couleur et nous demande de nous colorer.
   */
  private void receptionDemarre(int identSource, int identDest)
  {
    if (identDest == this.getIdentifiantProfondeur())
    {
      coloreParJeton = true;
      if (!voisinsTrackConnus) chercheRoutesBackTrack(); //si on n'a pas encore cherch les chemins de routage pour nous
      booleanVector.clear();
      setCouleursVoisinsSupp();
      //si on avait une couleur pas officielle valide, on l'essaye en priorit
      if ((couleur<=0)||(booleanVector.get(couleur)))
        couleur = getCouleurMaxPossible();
      else if (verbose) affiche("[-------Coloration non officielle utile----------------");
      booleanVector.set(couleur);
      if (couleur<=0) colorationRattee();
      else if (this.getIdentifiantProfondeur()<=0) receptionOK(identifiant); //backtrack termin et russi
      else
      {
        envoyerAuxFaibles(AlgoDMessage.MSG_COLORASYNCH1_NUMCOLOR,Integer.toString(couleur),identifiant);
        envoyerATrack(AlgoDMessage.MSG_COLORASYNCH1_DEMARRE,Integer.toString(couleur),this.getIdentifiantProfondeur());
        //envoyerAEnfantsPasTrack(AlgoDMessage.MSG_COLORASYNCH1_DEMARRE_SJ,Integer.toString(couleur),this.getIdentifiantProfondeur());
      }
    }
    else //routage
    {
      int numVoisTrack = -1;
      int maxVoisinsInf = -1;
      if (getIdentifiantProfondeur()-poids>identDest) numVoisTrack = numVoisinParent;
      else for (int cpt=nbEnfants()-1; cpt>=0; cpt--)
      {
        int idProf = getEnfantAt(cpt).getIdentifiantProfondeur();
        if ((idProf>maxVoisinsInf) && (idProf<=identDest))
        {
          numVoisTrack = numOfVoisin(getEnfantAt(cpt).getIdentifiant());
          maxVoisinsInf = idProf; //Minimum des fils infrieur ou gal  la destination
        }
      }
      if (numVoisTrack<0) numVoisTrack = this.numVoisinParent;
      envoyerAVoisin(AlgoDMessage.MSG_COLORASYNCH1_DEMARRE,"",identSource,numVoisTrack,identDest);
    }
  }

  private void receptionDemarre_SJ(int identSource, int colore, int identDest)
  {
    if (!coloreParJeton) //on n'est pas color par une couleur officielle
    {
      if (!voisinsTrackConnus) chercheRoutesBackTrack(); //si on n'a pas encore cherch les chemins de routage pour nous
      booleanVector.clear();
      setCouleursVoisinsSupp();
      couleur = getCouleurMaxPossible();
      booleanVector.set(couleur);
      if (couleur<=0) colorationRattee_SJ();
      else if (this.getIdentifiantProfondeur()<=0) receptionOK_SJ(identifiant); //backtrack termin et russi
      else
      {
        envoyerAuxFaibles(AlgoDMessage.MSG_COLORASYNCH1_NUMCOLOR_SJ,Integer.toString(couleur),identifiant);
        envoyerAuxFaibles(AlgoDMessage.MSG_COLORASYNCH1_DEMARRE_SJ,Integer.toString(couleur),identifiant);
        //envoyerAuxFaibles(AlgoDMessage.MSG_COLORASYNCH1_NUMCOLOR_SJ,Integer.toString(couleur),identifiant);
        //envoyerAuxEnfants(AlgoDMessage.MSG_COLORASYNCH1_DEMARRE_SJ,Integer.toString(couleur),this.getIdentifiantProfondeur());
        //envoyerATrack(AlgoDMessage.MSG_COLORASYNCH1_DEMARRE_SJ,Integer.toString(couleur),this.getIdentifiantProfondeur());
        //envoyerAEnfantsPasTrack(AlgoDMessage.MSG_COLORASYNCH1_DEMARRE_SJ,Integer.toString(couleur),this.getIdentifiantProfondeur());
      }
    }
  }

  /**
   * On ne peut pas se colorer, on back-traque.
   */
  private void colorationRattee()
  {
    if (verbose) affiche("--"+identifiant+"("+getIdentifiantProfondeur()+") backtrack---------");
    nbBackTrack++;
    coloreParJeton = false;
    if (isElu()) finColoration(false);
    else envoyerABackTrack(AlgoDMessage.MSG_COLORASYNCH1_NON,"",identifiant);
  }

  private void colorationRattee_SJ()
  {
    if (!coloreParJeton)
    {
      if (verbose) affiche("--"+identifiant+"("+getIdentifiantProfondeur()+") backtrack SJ---------");
      nbBackTrack_SJ++;
      coloreParJeton = false;
      if (!isElu()) envoyerABackTrack(AlgoDMessage.MSG_COLORASYNCH1_NON_SJ,"",identifiant);
    }
  }

}
