package algod;

/**
 * Ajoute la capacit d'lection  un sommet. Election du sommet ayant le plus de voisin,
 * en cas d'galit, lection du plus gros identifiant.
 * @author <A href=http://www.alapetite.fr.st>Alexandre Alapetite</A>, <A href=http://www.andujar.fr.st>Brice Andujar</A>, <A href=mailto:gregory.gontier@free.fr>Gregory Gontier</A>
 * @version 1.0
 */
abstract class SommetElection extends SommetIO
{
  static int nbMessagesElection = 0;

  /**
   * Tableau indiquant, parmis les sommets supposs du graphe, ceux qui participent.
   */
  private BooleanVector participant;

  /*
   * Valeur de l'acquitement reu.
   * @deprecated les messages ngatifs ne sont plus envoys
   */
  //private BooleanVector reponse;

  /**
   * Tableau indiquant, pour les sommets supposs du graphe, le nombre de rponses attendues.
   */
  private IntVector nbRep;

  /**
   * Tableau indiquant, pour les sommets supposs du graphe, le voisin qui relaie la communication.
   */
  IntVector route;

  /**
   * Nombre de sommets supposs dans le graphe.
   */
  private int nbSommetsPossibles;

  /**
   * Identifiant maximum des sommets candidats  l'lection.
   * -1 si aucun candidant ne s'est encore prsent.
   * @see #maxNbVoisins
   * @see #identElu
   */
  private int maxIdent;

  /**
   * Nombre de sommets du meilleur candidat  l'lection.
   * -1 si aucun candidant ne s'est encore prsent.
   * @see #maxIdent
   * @see #identElu
   */
  private int maxNbVoisins;

  /**
   * Identifiant du sommet lu. -1 si pas encore lu.
   * @see #maxIdent
   * @see #maxNbVoisins
   */
  int identElu;

  public SommetElection(int identifiant)
  {
    super(identifiant);
    this.nbSommetsPossibles = (identifiant+1) * 2;
    initialisationElection(nbSommetsPossibles);
  }

  public void destroy()
  {
    route.destroy();
    route = null;
    super.destroy();
  }

  protected void initialisationElection(int nbSomm)
  {
    this.nbSommetsPossibles = nbSomm;
    this.participant = new BooleanVector(nbSomm,false);
    //this.reponse = new BooleanVector(nbSomm,true);
    this.nbRep = new IntVector(nbSomm,Integer.MAX_VALUE);
    this.route = new IntVector(nbSomm,-1);
    this.maxIdent = -1;
    this.maxNbVoisins = -1;
    this.identElu = -1;
  }

  protected int ajoutVoisin(SommetVoisin unSommet)
  {
    int nv = super.ajoutVoisin(unSommet);
    route.set(unSommet.getIdentifiant(),nv); //enregistre la route pour le voisins, triviale
    return nv;
  }

  /**
   * Retourne si ce sommet est l'lu de son groupe.
   */
  public final boolean isElu()
  {
    return identElu == identifiant;
  }

  protected void demarreElection()
  {
    SommetIO.rndDelaiTransmission = 100;
    if (nbVoisins()<=0) affiche("#--- "+identifiant+" est isole--------------");
    else if (!participant.get(identifiant))
    {
      affiche("#--- "+identifiant+" debutte l'election-------------------------");
      acteCandidature();
    }
    else affiche("#--- "+identifiant+" ne sera pas elu-------------------------");
  }

  /**
   * Mthode invoque lorsque l'algorithme d'lection est termin.<BR>
   * Ce sommet est lu.
   */
  protected void finElection()
  {
    identElu = identifiant;
    nbRep.destroy();
    nbRep = null;
    participant.destroy();
    participant = null;
    //reponse.destroy();
    //reponse = null;
    affiche("#--- "+identifiant+"("+maxNbVoisins+") est elu --------------------------");
    envoyerATousVoisins(AlgoDMessage.MSG_ELECT_ELU,"",identifiant); //facultatif
    //simplement pour qu'en mode verbose, il n'y ait pas trop de messages concernant
    //la fin de l'lection. Ce n'est pas grave autrement, vue qu'il s'agit d'une diffusion,
    //que l'algo d'arbre couvrant qui suit est aussi une diffusion, et que les files sont FIFO.
    if (verbose) attend(1500);
    else attend(100);
  }

  /**
   * Dmarre un acte de candidature.
   */
  final void acteCandidature()
  {
    participant.set(identifiant);
    nbRep.set(identifiant,nbVoisins());
    maxNbVoisins = nbVoisins();
    maxIdent = identifiant;
    if (nbVoisins()<=0) finElection();
    else envoyerATousVoisins(AlgoDMessage.MSG_ELECT_IDENTIFIANT,Integer.toString(maxNbVoisins),maxIdent);
  }

  protected void traiteReception(AlgoDMessage adm, int numVoisin)
  {
    switch (adm.typeMessage)
    {
      case AlgoDMessage.MSG_ELECT_IDENTIFIANT : //candidature  l'lection
        nbMessagesElection++;
        receptionIdentifiant(adm.identSommetSource,Integer.parseInt(adm.contenu),numVoisin);
        break;
      case AlgoDMessage.MSG_ELECT_ELU : //identifiant de l'lu
        nbMessagesElection++;
        receptionElu(adm.identSommetSource,numVoisin);
        break;
      case AlgoDMessage.MSG_ELECT_OUI : //d'accord pour ce candidat
        nbMessagesElection++;
        receptionAccuse(adm.identSommetSource,numVoisin,true);
        break;
      /*case AlgoDMessage.MSG_ELECT_NON : //pas d'accord pour ce candidat
        nbMessagesElection++;
        receptionAccuse(adm.identSommetSource,numVoisin,false);
        break;*/
      default : super.traiteReception(adm,numVoisin);
    }
  }

  /**
   * @return si le candidat possdant ce couple de caractristiques est meilleur
   * que le candidat actuellement retenu.
   * @param nbDeVoisins Nombre de voisins du candidat.
   * @param id Identifiant du candidat (pour un ordre total en cas d'galit sur le nombre de voisins).
   */
  private boolean isMeilleur(int nbDeVoisins, int id)
  {
    return (nbDeVoisins>maxNbVoisins) || ((nbDeVoisins==maxNbVoisins) && (id>maxIdent));
  }

  /**
   * Traitement d'un message de diffusion.
   */
  private void receptionIdentifiant(int sommet_source, int nbDeVoisins, int numVoisin)
  {
    if (identElu>=0) return; //l'lection est termine
    else if (participant.get(sommet_source))
    {
      if (maxIdent==sommet_source)
        envoyerAVoisin(AlgoDMessage.MSG_ELECT_OUI,"",sommet_source,numVoisin);
      //else envoyerAVoisin(AlgoDMessage.MSG_ELECT_NON,"",sommet_source,numVoisin);
    }
    else
    {
      if (!participant.get(identifiant)) //si on ne participe pas encore  l'lection
        if ((nbVoisins() > nbDeVoisins) ||
            ((nbVoisins() == nbDeVoisins) && (identifiant > sommet_source))
           ) acteCandidature(); //dmarrage ventuel de notre candidature
        else participant.set(identifiant);
      participant.set(sommet_source);
      if (isMeilleur(nbDeVoisins,sommet_source)) //si le nouveau candidat est meilleur
      {
        maxNbVoisins = nbDeVoisins;
        maxIdent = sommet_source;
        nbRep.set(sommet_source,nbVoisins()-1);
        if (!route.isSet(sommet_source)) route.set(sommet_source,numVoisin);
        if (nbVoisins()<=1)
          envoyerAVoisin(AlgoDMessage.MSG_ELECT_OUI,"",sommet_source,numVoisin);
        else //diffuse la candidature //TODO: ne pas envoyer au candidat
          envoyerATousVoisinsSauf(AlgoDMessage.MSG_ELECT_IDENTIFIANT,Integer.toString(nbDeVoisins),sommet_source,numVoisin);
      }
      //else envoyerAVoisin(AlgoDMessage.MSG_ELECT_NON,"",sommet_source,numVoisin);
    }
  }

  /**
   * Traitement d'un message d'accus-rception.
   */
  private void receptionAccuse(int sommet_source, int numVoisin, boolean valeur)
  {
    if (identElu>=0) return; //l'lection est termine
    nbRep.dec(sommet_source);
    //reponse.and(sommet_source,valeur);
    if (nbRep.get(sommet_source)==0)
    {
      if (sommet_source==identifiant)
      {
        if (/*reponse.get(identifiant) &&*/ (identifiant==maxIdent) && (identElu<0)) finElection();
        //else affiche("#---"+identifiant+" n'est pas elu");
      }
      else if (route.get(sommet_source)<0) afficheErreur("/!\\ route non definie");
      else /*if (reponse.get(sommet_source))*/ envoyerAVoisin(AlgoDMessage.MSG_ELECT_OUI,"",sommet_source,route.get(sommet_source),sommet_source);
      //else envoyerAVoisin(AlgoDMessage.MSG_ELECT_NON,"",sommet_source,route.get(sommet_source));
    }
    else if (nbRep.get(sommet_source)<0) afficheErreur("/!\\ trop de reponses");
  }

  /**
   * Rception d'un message contenant l'identifiant de l'lu.
   */
  private void receptionElu(int sommet_source, int numVoisin)
  {
    if (identElu<0) //on ne connaissait pas encore l'lu
    {
      this.identElu = sommet_source;
      nbRep.destroy();
      nbRep = null;
      participant.destroy();
      participant = null;
      //reponse.destroy();
      //reponse = null;
      envoyerATousVoisinsSauf(AlgoDMessage.MSG_ELECT_ELU,"",sommet_source,numVoisin);
    }
  }

  public String toString()
  {
    return super.toString()+"; elu="+identElu;
  }

}
