package algod;

/**
 * Ajoute les fonctionnalits de communication  un Sommet.
 * @author <A href=http://www.alapetite.fr.st>Alexandre Alapetite</A>, <A href=http://www.andujar.fr.st>Brice Andujar</A>, <A href=mailto:gregory.gontier@free.fr>Gregory Gontier</A>
 * @version 1.0
 */
abstract class SommetIO extends SommetAbstract
{
  static int rndDelaiTransmission = 10;

  /**
   * File d'attente des messages provenant d'autres sommets.
   */
  protected AlgoDMessageFIFO algoDMessageFIFO;

  public SommetIO(int identifiant)
  {
    super(identifiant);
    algoDMessageFIFO = new AlgoDMessageFIFO();
  }

  public void destroy()
  {
    algoDMessageFIFO.clear();
    algoDMessageFIFO = null;
    super.destroy();
  }

  private void envoyerASommet(AlgoDMessage adm, SommetVoisin sommetDestination)
  {
    if (verbose) attend((long)(Math.random()*rndDelaiTransmission));
    if (verbose) affiche("->"+identifiant+" envoie a "+sommetDestination.getIdentifiant()+":"+adm.toString());
    sommetDestination.recevoir(adm);
  }

  final void envoyerASommet(char typeMessage, String contenu, int identSommetSource, SommetVoisin sommetVoisin, int identSommetDestination)
  {
    AlgoDMessage adm = new AlgoDMessage(typeMessage,contenu,identSommetSource,identifiant,identSommetDestination);
    envoyerASommet(adm,sommetVoisin);
  }

  final void envoyerASommet(char typeMessage, String contenu, int identSommetSource, SommetVoisin sommetVoisin)
  {
    envoyerASommet(typeMessage,contenu,identSommetSource,sommetVoisin,sommetVoisin.getIdentifiant());
  }

  /**
   * Envoie un message de type typeMessage, provenant du sommet identSommetSource
   * que l'on achemine par le voisin numVoisin pour le sommet identSommetDestination.
   */
  protected final void envoyerAVoisin(char typeMessage, String contenu, int identSommetSource, int numVoisin, int identSommetDestination)
  {
    envoyerASommet(typeMessage,contenu,identSommetSource,getVoisinAt(numVoisin),identSommetDestination);
  }

  protected final void envoyerAVoisin(char typeMessage, String contenu, int identSommetSource, int numVoisin)
  {
    envoyerASommet(typeMessage,contenu,identSommetSource,getVoisinAt(numVoisin));
  }

  /**
   * Envoie un message  tous les voisins.
   */
  protected final void envoyerATousVoisins(char typeMessage, String contenu, int identSommetSource)
  {
    for (int cpt=nbVoisins()-1; cpt>=0; cpt--)
      envoyerAVoisin(typeMessage,contenu,identSommetSource,cpt);
  }

  /**
   * Envoie un message  tous les voisins sauf un.
   * @param numVoisin le numro du voisin  ignorer (pas son identifiant).
   */
  protected final void envoyerATousVoisinsSauf(char typeMessage, String contenu, int identSommetSource, int numVoisin)
  {
    for (int cpt=nbVoisins()-1; cpt>=0; cpt--)
      if (cpt!=numVoisin)
        envoyerAVoisin(typeMessage,contenu,identSommetSource,cpt);
  }

  /**
   * Indique que le sommet n'a plus de message  traiter dans la file d'attente.
   */
  public final boolean isFini()
  {
    return algoDMessageFIFO.empty();
  }

  /**
   * Appelle les mthodes appropries en fonction du message  traiter
   * aprs un parsage ventuel.
   */
  protected void traiteReception(AlgoDMessage adm, int numVoisin)
  {
    afficheErreur("/!\\ Sommet "+identifiant+" recoit message inconnu : "+adm.toString());
  }

  /**
   * Ajoute un message provenant d'un autre sommet  la file des messages. <BR>
   * La file FIFO est dj synchronized.
   * @see #algoDMessageFIFO
   */
  public final void recevoir(AlgoDMessage adm)
  {
    algoDMessageFIFO.push(adm);
    //affiche("<-"+identifiant+" recoit "+adm.toString());
  }

  public String toString()
  {
    return super.toString()+"; messages="+(!isFini());
  }

}
