package algod;

/**
 * Ajoute la fonctionnalit de numrotation en profondeur inverse de l'arbre couvrant.
 * @author <A href=http://www.alapetite.fr.st>Alexandre Alapetite</A>, <A href=http://www.andujar.fr.st>Brice Andujar</A>, <A href=mailto:gregory.gontier@free.fr>Gregory Gontier</A>
 * @version 1.0
 */
class SommetNumProfondeur extends SommetArbreCouvrant
{
  static int nbMessagesNumProfondeur = 0;
  /**
   * Nouvel identifiant aprs l'arbre couvrant, qui correspond  une
   * numrotation en profondeur inverse.
   */
  private int identifiantProfondeur;

  public SommetNumProfondeur(int identifiant)
  {
    super(identifiant);
    this.identifiantProfondeur = -1;
  }

  /**
   * Accs au nouvel identifiant aprs l'arbre couvrant, qui correspond  une
   * numrotation en profondeur inverse.
   */
  public final int getIdentifiantProfondeur()
  {
    return this.identifiantProfondeur;
  }

  public void demarreNumProfondeur()
  {
    affiche("--- "+identifiant+" lance la numerotation en profondeur --------");
    receptionSetIdentifiantProfondeur(this.poids);
    attend(1000);
    finNumProfondeur();
  }

  protected void finNumProfondeur()
  {
    affiche("--- "+identifiant+" fin de numerotation en profondeur ---------");
  }

  /**
   * Termine la mthode hrite de fin d'arbre couvrant pour dmarrer
   * la numrotation en profondeur.
   */
  protected final void finArbreCouvrant()
  {
    super.finArbreCouvrant();
    demarreNumProfondeur();
  }

  protected void traiteReception(AlgoDMessage adm, int numVoisin)
  {
    switch (adm.typeMessage)
    {
      case AlgoDMessage.MSG_NUMPROF_IDPROF :
        nbMessagesNumProfondeur++;
        receptionIdentifiantProfondeur(numVoisin,Integer.parseInt(adm.contenu));
        break;
      case AlgoDMessage.MSG_NUMPROF_SETIDPROF :
        nbMessagesNumProfondeur++;
        receptionSetIdentifiantProfondeur(Integer.parseInt(adm.contenu));
        break;
      default : super.traiteReception(adm, numVoisin);
    }
  }

  /**
   * Rcepetion par un voisin de son identifiant profondeur.
   */
  private void receptionIdentifiantProfondeur(int numVoisin,int idPr)
  {
    getVoisinAt(numVoisin).setIdentifiantProfondeur(idPr);
  }

  /**
   * Rception de notre identifiant profondeur par notre pre.
   */
  private void receptionSetIdentifiantProfondeur(int idPr)
  {
    this.identifiantProfondeur = idPr;
    idPr--;
    int maxim = nbEnfants();
    for (int cpt=0; cpt<maxim; cpt++) //envoie  tous les enfants
    {
      getEnfantAt(cpt).setIdentifiantProfondeur(idPr); //sauve en local l'idenfifiant profondeur des enfants
      envoyerAEnfant(AlgoDMessage.MSG_NUMPROF_SETIDPROF,""+idPr,identifiant,cpt);
      idPr-=poidsEnfants.get(cpt);
    }
    //transmission  tous nos voisins (sauf le pre) de notre identifiant profondeur
    envoyerATousVoisinsSauf(AlgoDMessage.MSG_NUMPROF_IDPROF,Integer.toString(identifiantProfondeur),identifiant,numVoisinParent);
  }

  public String toString()
  {
    return super.toString()+"; identProfond="+identifiantProfondeur;
  }

}
