package algod;

import java.io.IOException;

/**
 * Problme de k coloration en algorithmique distribue.
 * @author <A href=http://www.alapetite.fr.st>Alexandre Alapetite</A>, <A href=http://www.andujar.fr.st>Brice Andujar</A>, <A href=mailto:gregory.gontier@free.fr>Gregory Gontier</A>
 * @version 1.0
 */
public class TestAlgoD extends Graphe
{
  /**
   * Indique si le programme doit continuer  s'excuter.
   * La squence de destruction, avec arrt propre des threads dmarre lorsque <CODE>tourne</CODE> passe  faux.
   */
  boolean tourne;

  public TestAlgoD(int nbSommets, int nbColorations, double densite, boolean graphique, boolean verbose, long randomSeed)
  {
    super(nbSommets,nbColorations,graphique);
    affiche("parametres:"+nbSommets+","+nbColorations+","+densite+","+graphique+","+verbose+","+randomSeed);
    Sommet.verbose = verbose;
    this.tourne = true;
    super.genereArretesAlleatoires(densite,randomSeed);
    super.start();
    while (tourne)
      try
      {
        traiteCommande(System.in.read());
      }
      catch (IOException ioe)
      {
        afficheErreur(ioe.toString());
      }
    affiche("Arret de l'algo demande");
    super.destroy();
  }

  public TestAlgoD(String fileName, int nbColorations, boolean graphique, boolean verbose)
  {
    super(fileName,nbColorations,graphique);
    affiche("parametres:xml,"+nbColorations+","+graphique+","+verbose);
    Sommet.verbose = verbose;
    this.tourne = true;
    super.start();
    while (tourne)
      try
      {
        traiteCommande(System.in.read());
      }
      catch (IOException ioe)
      {
        afficheErreur(ioe.toString());
      }
    affiche("Arret de l'algo demande");
    super.destroy();
  }

  final void finDesinscription()
  {
    super.finDesinscription();
    affiche("Le programme s'est fini correctement");
    System.exit(0);
  }

  /**
   * Traite une commande entre au clavier sur l'entre standard.
   * <UL>
   * <LI>q : demande un arrt du programme et de tous les threads.
   * <LI>i : affiche des informations comme le nombre de messages, de back-tracks, de couleurs.
   * <LI>s : affiche le <CODE>toString()</CODE>
   * <LI>a : teste si l'algo est arrt ou pas (s'il reste des messages dans les files).
   * </UL>
   */
  public void traiteCommande(int commande)
  {
    switch (commande)
    {
      case 'q' :
        tourne = false;
        break;
      case 'i' :
        super.informations();
        break;
      case 's' :
        affiche(super.toString());
        break;
      case 't' :
        super.test();
        break;
      case 'a' :
        affiche("Messages en attente = "+super.isActive());
        break;
      case ' ' :
      case 10 :
      case 13 :
        break;
      default : afficheErreur("Commande inconnue : ASCII("+commande+")="+((char)commande));
    }
  }

  /**
   * Point d'entre pour tester les algorithmes de <CODE>Sommet</CODE>. <BR>
   * Supporte des paramtres, comme ? pour l'aide et<BR>
   * nombre de couleurs[ ]nombre de sommets[ ]proba d'arc[ ](verboseEnvoie y-n)[](graine random)
   */
  public static void main(String[] args)
  {
    if (args.length<1)
    {
      //coloration, nombre de sommets, proba d'arc, graine de random
      //new TestAlgoD(4,2,0.3,true,true,50000); //sommet 2 au centre TestAlgoD(2,4,0.3,50000)
      new TestAlgoD(15,3,0.25,true,true,50000); //test normal TestAlgoD(4,15,0.3,50000)
      //new TestAlgoD(7,4,0.8,true,true,50000); //backtrack
      //new TestAlgoD(25,3,0.1,true,true,System.currentTimeMillis()); //alatoire
    }
    else if (args[0].equals("?"))
    {
      affiche("aide TestAlgoD:");
      affiche("nombre de sommets[ ]nombre de couleurs[ ]proba d'arc[ ](graphique y-n)[](verboseEnvoie y-n)[](graine random)");
      affiche("xml nomfichier[ ]nombre de couleurs[ ](graphique y-n)[](verboseEnvoie y-n)");
    }
    else if (args[0].equalsIgnoreCase("xml"))
    {
      affiche("NbParams="+args.length+":"+args[0]+"|"+args[1]+"|"+Integer.parseInt(args[2]));
      boolean graphique = (args.length>=4) ? args[3].equalsIgnoreCase("y") : false;
      boolean verbose = (args.length>=5) ? args[4].equalsIgnoreCase("y") : false;
      new TestAlgoD(args[1],Integer.parseInt(args[2]),graphique,verbose);
    }
    else
    {
      boolean graphique = (args.length>=4) ? args[3].equalsIgnoreCase("y") : false;
      boolean verbose = (args.length>=5) ? args[4].equalsIgnoreCase("y") : false;
      long graine = (args.length>=6) ? Long.parseLong(args[5]) : System.currentTimeMillis();
      new TestAlgoD(Integer.parseInt(args[0]),Integer.parseInt(args[1]),Float.parseFloat(args[2]),graphique,verbose,graine);
    }
  }

}
