package alxXML;

import java.io.*;
import java.util.Hashtable;
import java.util.Enumeration;

/**
 * Description : Parser XML permettant la lecture, l'criture de documents XML.
 * @author <A href=http://www.alapetite.fr.st target=_blank>Alexandre Alapetite</A>
 * @version 1.0 (06/2001)
 */
public class AlxXMLdocument extends Object
{
  /** Attributs de ce document XML. */
  private Hashtable htAttributs = new Hashtable(); //de String
  /** Element principal de ce document XML. */
  protected AlxXMLelement alxXMLelement;
  /** Table de messages d'erreur. */
  protected static String erreurs[] = new String[]
  {
    "Ce n'est pas un fichier XML",
    "Fin prmature du fichier XML",
    "Caractres inattendus",
    "Erreur de dcodage du fichier XML",
    "Balise fermante introuvable"
  };

  /**
   * Cration d'un nouveau document avec le nom de l'lment principal.
   * L'attribut encoding a une valeur par dfaut de ISO-8859-1.
   * L'attribut version a une valeur par dfaut de 1.0 (c'est la version du XML utilis dans le fichier (seule la 1.0 est gre)).
   */
  public AlxXMLdocument(String nom)
  {
    super();
    alxXMLelement = new AlxXMLelement(nom);
    alxXMLelement.setPere(null);
    addAttribut("version","1.0");
    addAttribut("encoding","ISO-8859-1");
  }

  /** @return l'lment principal du document XML. */
  public AlxXMLelement getMainElement()
  {
    return alxXMLelement;
  }

  /**
   * Ajoute ou met  jour un attribut de ce document XML.
   * @param attribut nom de l'attribut
   * @param valeur valeur de l'attribut spcifi
   */
  public void addAttribut(String attribut, String valeur)
  {
    htAttributs.remove(attribut);
    htAttributs.put(attribut,valeur);
  }

  /** @return la valeur de l'attribut de ce nom s'il existe, une chane vide sinon. */
  public String getAttribut(String attribut)
  {
    String sTemp = (String)htAttributs.get(attribut);
    return sTemp==null ? new String() : sTemp;
  }

  /**
   * Dbut du dcodage du document XML. L il s'agit de la lecture de la 1re ligne.
   * Mthode appele dans #decodeXMLdocument.
   * @param streamTokenizer un <CODE>StreamTokenizer</CODE> pas encore utilis et branch sur un <CODE>BufferedReader</CODE> branch sur le fichier XML  dcoder.
   * @see #decodeXMLdocument
   */
  protected void decodeEntete(StreamTokenizer streamTokenizer) throws IOException
  {
    if (!(
          streamTokenizer.nextToken()=='<' &&
          streamTokenizer.nextToken()=='?' &&
          streamTokenizer.nextToken()==StreamTokenizer.TT_WORD && streamTokenizer.sval.equalsIgnoreCase("xml")
          )) throw new AlxXMLioException(streamTokenizer.lineno(),"<?xml attendu en dbut de ligne");

    boolean interro=false; //verification du point d'interrogation de fin
    while (streamTokenizer.nextToken()!='>')
    {
      if (streamTokenizer.ttype==StreamTokenizer.TT_EOF) throw new AlxXMLioException(streamTokenizer.lineno(),AlxXMLdocument.erreurs[1]);
      else if (streamTokenizer.ttype==StreamTokenizer.TT_WORD)
      {
        String attribut = streamTokenizer.sval;
          if (streamTokenizer.nextToken()!='=' || streamTokenizer.nextToken()!='"') //attend un [=] puis ["****"]
            throw new AlxXMLioException(streamTokenizer.lineno(),AlxXMLdocument.erreurs[2]);
          else this.addAttribut(attribut,streamTokenizer.sval);
      }
      else if (streamTokenizer.ttype=='?') interro=true;
    }
    if (!interro) throw new AlxXMLioException(streamTokenizer.lineno(),"?> attendu en fin de ligne");
  }

  /**
   * Mthode principale du dcodage du document XML.
   * Mthode appele dans {@link #readFromFile(String) readFromFile}.
   * @param bufferedReader un <CODE>BufferedReader</CODE> pas encore utilis et branch sur le fichier XML  dcoder.
   * @see #readFromFile readFromFile<BR>
   * @see #decodeEntete decodeEntete appel en premier<BR>
   * @see AlxXMLelement#decodeXMLelement decodeXMLelement appele en second pour l'lment principal du document XML
   */
  protected void decodeXMLdocument(BufferedReader bufferedReader) throws IOException
  {
    StreamTokenizer streamTokenizer = new StreamTokenizer(bufferedReader);
    streamTokenizer.wordChars('_','_'); //le "_" fait parti du mot
    streamTokenizer.ordinaryChar('/');
    //streamTokenizer.quoteChar('"'); //dj par dfaut (considre que ["*****"] est un seul mot
    decodeEntete(streamTokenizer);
    if (!(streamTokenizer.nextToken()=='<'))
      throw new AlxXMLioException(streamTokenizer.lineno(),"< attendu");
    else this.alxXMLelement.decodeXMLelement(streamTokenizer);
  }

  /**
   * Lit un fichier XML et le dcode.
   * Appelle {@link #decodeXMLdocument decodeXMLdocument}.
   * @param adresse l'adresse du fichier XML.
   * @throws AlxXMLioException s'il y a eu un problme d'entre-sortie ou si le fichier XML n' pas t compris.
   */
  public void readFromFile(String adresse) throws AlxXMLioException
  {
    FileReader fileReader=null;
    BufferedReader bufferedReader=null;
    try
    {
      fileReader = new FileReader(adresse);
      bufferedReader = new BufferedReader(fileReader);
      decodeXMLdocument(bufferedReader);
    }
    catch (AlxXMLioException axie) {throw axie; }
    catch (IOException ioe) { throw new AlxXMLioException(AlxXMLdocument.erreurs[3]+"="+ioe.toString()); }
    finally
    {
      try
      {
        if (bufferedReader!=null) bufferedReader.close();
        if (fileReader!=null) fileReader.close();
      }
      catch (IOException ioe) { throw new AlxXMLioException(AlxXMLdocument.erreurs[3]+"="+ioe.toString()); }
    }
  }

  /**
   * Ouvre un nouveau fichier XML et l'encode.
   * Appelle {@link #toString toString}.
   * @param adresse l'adresse du fichier XML.
   * @throws AlxXMLioException s'il y a eu un problme d'entre-sortie.
   */
  public void saveToFile(String adresse) throws AlxXMLioException
  {
    FileWriter fileWriter=null;
    BufferedWriter bufferedWriter=null;
    try
    {
      fileWriter = new FileWriter(adresse);
      bufferedWriter = new BufferedWriter(fileWriter);
      bufferedWriter.write(this.toString());
    }
    catch (IOException ioe) {  throw new AlxXMLioException(AlxXMLdocument.erreurs[3]+"="+ioe.toString()); }
    finally
    {
      try
      {
        if (bufferedWriter!=null) bufferedWriter.close();
        if (fileWriter!=null) fileWriter.close();
      }
      catch (IOException ioe) {  throw new AlxXMLioException(AlxXMLdocument.erreurs[3]+"="+ioe.toString()); }
    }
  }

  /** @return le document XML courant au format texte XML. */
  public String toString()
  {
    String result="<?xml";
    Enumeration eTemp = htAttributs.keys();
    while (eTemp.hasMoreElements())
    {
       String clef = (String)eTemp.nextElement();
       String valeur = (String)htAttributs.get(clef);
       result+=" "+clef+"=\""+valeur+"\"";
    }
    result+="?>\n";
    result+=alxXMLelement.toString()+"\n"; //affiche tous les sous-lments
    return result;
  }

}
