package alxXML;

import java.util.Vector;
import java.util.Hashtable;

/**
 * Description : Gre les sous-lments d'un {@link AlxXMLelement AlxXMLelement}.
 * @author <A href=http://www.alapetite.fr.st target=_blank>Alexandre Alapetite</A>
 * @version 1.0 (06/2001)
 */
public class AlxXMLelementList extends Object
{
  /** Vecteur contenant tous les <CODE>AlxXMLelement</CODE>. */
  private Vector vComplet = new Vector(); //de alxXMLelement
  /** Table de hashage contenant des couples <CODE>String</CODE>-<CODE>Vector</CODE> de <CODE>AlxXMLelement</CODE> de mme nom. */
  private Hashtable htFamilleElement = new Hashtable(); //de Vector de AlxXMLelement

  public AlxXMLelementList()
  {
    super();
  }

  /** @return le <CODE>AlxXMLelement</CODE>  cet indice. */
  public AlxXMLelement get(int index)
  {
    return (AlxXMLelement)vComplet.get(index);
  }

  /** @return l'indice de ce <CODE>AlxXMLelement</CODE>. */
  public int indexOf(AlxXMLelement alxXMLelement)
  {
    return vComplet.indexOf(alxXMLelement);
  }

  /**
   * Ajoute un nouvel <CODE>AlxXMLelement</CODE>.
   * En fait, appelle {@link #addInFamille addInFamille} avec comme nom de famille le nom de l'<CODE>AlxXMLelement</CODE>.
   */
  public boolean add(AlxXMLelement alxXMLelement)
  {
    return addInFamille(alxXMLelement,alxXMLelement.getNom());
  }

  /**
   * Supprime un <CODE>AlxXMLelement</CODE>.
   * @return vrai si l'lment  supprimer  t trouv, faux sinon.
   */
  public synchronized boolean remove(AlxXMLelement alxXMLelement)
  {
    boolean trouve=false;
    Vector vTemp = (Vector)htFamilleElement.get(alxXMLelement.getNom());
    if (vTemp!=null) trouve=vTemp.removeElement(alxXMLelement);
    trouve = trouve && vComplet.remove(alxXMLelement);
    return trouve;
  }

  /** @return le nombre de <CODE>AlxXMLelement</CODE> stoqus. */
  public int size()
  {
    return vComplet.size();
  }

  /**
   * Lorsque plusieurs <CODE>AlxXMLelement</CODE> ont le mme nom, ils sont numrots au sein d'un famille de <CODE>AlxXMLelement</CODE> ayant le mme nom.
   * @param index indice souhait.
   * @param famille nom des <CODE>AlxXMLelement</CODE> cherchs.
   * @return le index<SUP>ime</SUP> <CODE>AlxXMLelement</CODE> ayant ce nom, null si le nom n'a pas t trouv ou si l'indice n'est pas valide.
   */
  public synchronized AlxXMLelement getInFamille(int index, String famille)
  {
    Vector vTemp = (Vector)htFamilleElement.get(famille);
    return vTemp==null ? null : (AlxXMLelement)vTemp.get(index);
  }

  /**
   * Lorsque plusieurs <CODE>AlxXMLelement</CODE> ont le mme nom, ils sont numrots au sein d'un famille de <CODE>AlxXMLelement</CODE> ayant le mme nom.
   * @param alxXMLelement <CODE>AlxXMLelement</CODE> cherch dans cette famille.
   * @param famille nom de l'<CODE>AlxXMLelement</CODE> cherch.
   * @return l'index de l'<CODE>AlxXMLelement</CODE> cherch au sein de cette famille, -1 s'il n'a pas t trouv.
   */
  public synchronized int indexOfInFamille(AlxXMLelement alxXMLelement, String famille)
  {
    Vector vTemp = (Vector)htFamilleElement.get(famille);
    return vTemp==null ? -1 : vTemp.indexOf(alxXMLelement);
  }

  /**
   * Ajoute un nouvel <CODE>AlxXMLelement</CODE>.
   * Mthode appele par {@link #add add} avec comme nom de famille le nom de l'<CODE>AlxXMLelement</CODE>.
   * @return vrai si l'opration s'est bien passe, faux sinon
   */
  protected synchronized boolean addInFamille(AlxXMLelement alxXMLelement, String famille)
  {
    if (!htFamilleElement.containsKey(famille)) htFamilleElement.put(famille,new Vector());
    return vComplet.add(alxXMLelement) && ((Vector)htFamilleElement.get(famille)).add(alxXMLelement);
  }

  /**
   * @return le nombre de <CODE>AlxXMLelement</CODE> dans la famille de ce nom.
   */
  public int sizeFamille(String famille)
  {
    Vector vTemp = (Vector)htFamilleElement.get(famille);
    return vTemp==null ? 0 : vTemp.size();
  }

  /**
   * @return le nombre de familles diffrents, soit le nombre de <CODE>AlxXMLelement</CODE> ayant des noms diffrents.
   */
  public int sizeListFamille()
  {
    return htFamilleElement.size();
  }

  /**
   * Mthode appele par {@link AlxXMLelement#toString AlxXMLelement.toString()}.
   * Appelle les mthodes {@link AlxXMLelement#toString AlxXMLelement.toString()} de tous les <CODE>AlxXMLelement</CODE> stoqus.
   * @return l'ensemble des <CODE>AlxXMLelement</CODE> au format texte XML.
   */
  public String toString()
  {
    String sTemp="";
    for (int cpt=0; cpt<vComplet.size(); cpt++) sTemp+=((AlxXMLelement)vComplet.get(cpt)).toString()+'\n';
    return sTemp;
  }
}
