package grapheanim;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.math.*;
import java.util.Vector;
import java.util.Properties;

/** Thread permettant le droulement automatique d'un algorithme. */
public class AlgoRun extends Thread
{
  private Graphics g;
  private DynamicGraphData stockage;
  private boolean arret;
  private Scrollbar avancement;
  private Button pauseB, backB, runB;
  private TextField avancementMin;
  private int vitesse, largeurScreen, hauteurScreen;
  private Image offscreen;
  private TextArea commentaire, algo;
  /** indique si l'algo en cours d'excution a besoin d'une arrive */
  private boolean uneArrivee;

  AlgoRun(DynamicGraphData s, Graphics graphics, Scrollbar scroll, Button b1, Button b2,
      Button b3, TextField t1, int v, Image o, int la, int ha, TextArea tt1, TextArea tt2)
  {
    stockage = s;
    g = graphics;
    avancement = scroll;
    backB = b1;
    pauseB = b2;
    runB = b3;
    avancementMin = t1;
    vitesse = v;
    offscreen = o;
    largeurScreen = la;
    hauteurScreen = ha;
    commentaire = tt1;
    algo = tt2;
  }

  public void stopper() { arret = true; }

  public void run()
  {
    String algorithme = stockage.getAlgo();
    uneArrivee=(algorithme.equals("Flot")) ||
               (algorithme.equals("A*")) ||
               (algorithme.equals("A")) ||
               (algorithme.equals("Lexico"));
    algorithme = null;

    arret = false;
    for (int i=stockage.getCurrentStepIndex();(!arret)&&(i<avancement.getMaximum());i++)
    {
      next();
      if (vitesse>0)
        try { sleep(vitesse * 200); }
        catch (InterruptedException e) { System.out.println(e); }
    }
    if (!arret)
    {
      pauseB.setEnabled(false);
      backB.setEnabled(true);
      runB.setEnabled(false);
    }
  }

  public void previous()
  {
    if (avancement.getMinimum() < stockage.getCurrentStepIndex())
    {
      stockage.gotoPreviousStep();
      avancement.setValue(avancement.getValue()-1);
      avancementMin.setText(String.valueOf(avancement.getValue()+1));
      commentaire.append(stockage.getStepAt(stockage.getCurrentStepIndex()).getCommentaire());
      algo.append(stockage.getStepAt(stockage.getCurrentStepIndex()).getAlgo());
      afficher();
    }
  }

  public void next()
  {
    if (avancement.getMaximum()-1 > stockage.getCurrentStepIndex())
    {
      stockage.gotoNextStep();
      avancement.setValue(avancement.getValue()+1);
      avancementMin.setText(String.valueOf(avancement.getValue()+1));
      commentaire.append(stockage.getStepAt(stockage.getCurrentStepIndex()-1).getCommentaire());
      algo.append(stockage.getStepAt(stockage.getCurrentStepIndex()-1).getAlgo());
      afficher();
    }
  }

  private void afficher()
  {
    Graphics og = offscreen.getGraphics();
    og.setClip(0,0,largeurScreen, hauteurScreen);
    og.setColor(Color.lightGray);
    og.fillRect(0,0, largeurScreen, hauteurScreen);
    stockage.afficher(og,uneArrivee,false);
    g.drawImage(offscreen, 0, 0, null);
    og.dispose();
  }

  void setVitesse(int v) { vitesse = v; }

}
