package grapheanim;

import java.awt.*;
import java.util.Vector;

/**
 * Description : Algorithme permettant de trouver un couplage dans un graphe bi-partie.
 * @author Delphine Lara-Martil & Nicolas Borde
 * @version 1.0
 */
public class Biparti extends Methode_Generale
{
  private int test=0;
  private String chaine;
  private String chainefinale;
  private DynamicGraphData DYN;
  private GrapheData Gd;

  public Biparti(DynamicGraphData dgd)
  {
    super(dgd);
    DYN = new DynamicGraphData();
    DYN=dgd;
    Gd= new GrapheData();
    Gd=dgd.getInitGraph();
    chaine="";
    chainefinale="";
  }

  /**
   * Mthode utilise pour rechercher un couplage "simple"
   * on parcourt les successeurs d'un sommet et on marque le premier successeur
   * de ce sommet non encore marqu, on rajoute l'arete entre le sommet et son successeur
   * dans le couplage.
   */
  public void Exploration(Sommet z)
  {
    String algo="";
    String commentaire="";
    algo="\nSommet visit : "+z.getNom();
    DynamicData DD=new DynamicData();
    //on recupre les successeurs du sommet z
    Vector success= new Vector();
    success=z.getAllSuccesseurs();
    int comp=0;boolean test=true;
    String nom="";
    //on parcourt le tableau de successeurs
    while(comp<success.size()&&test)
    {
      //recupration du nom du successeur
      nom=((Successeur)(success.elementAt(comp))).getNom();
      String v="";
      int i=0;
      //rcupration du sommet ayant ce nom
      while(!v.equals(nom))
      {
        v=((Sommet)Gd.sommet.elementAt(i)).getNom();
        i++;
      }
      i--;
      //si ce sommet appartient dj au couplage on avance dans le tableau
      if(((Sommet)Gd.sommet.elementAt(i)).getCouple()==true)
        comp++;
      //sinon on marque ce sommet
      else
      {
        test=false;
        ((Sommet)Gd.sommet.elementAt(i)).setCouple(true);}
      }

      if(comp<success.size())
      {
        //on a trouv un sommet non marqu
        chaine += z.getNom();
        //on colore le sommet sur lequel s'est effectue la recherche
        SauverCouleurSommet(Color.blue,z.getNom());
        String nom2=z.getNom();
        String v="";
        int i=0;
        while(!v.equals(nom2))
        {
          v=((Sommet)Gd.sommet.elementAt(i)).getNom();
          i++;
        }
        i--;
        ((Sommet)Gd.sommet.elementAt(i)).setCouple(true);
        chaine += nom;
        algo+="\nVoisin de "+z.getNom()+" choisi : "+nom;
        DD.setAlgo(algo);
        DD.setCommentaire(commentaire);
        //on colore son voisin choisi
        SauverCouleurSommet(Color.blue,nom);
        // on colore l'arte entre ces deux points car elle amliore le couplage
        SauverCouleurArrete(DD,Color.white,z.getNom(),nom);
        SauverCouleurArrete(DD,Color.white,nom,z.getNom());
        get_DGD().addStep(DD);
      }
   }

   /** Mthode d'affichage de la chaine amliorante choisie pour maximiser le couplage.  */
  public void afficher()
  {
    String algo="";
    algo+="\nOn amliore le flot en prenant une chaine amliorante\n";
    algo+="Les artes mauves sont les artes ne faisant plus partie\ndu couplage\n";
    DynamicData DD=new DynamicData();
    //on colore en violet les artes qui vont etre supprimes du couplage
    //on colore en vert les artes ajoutes
    //on colore en violet les sommets ajouts ou modifis du couplage initial

    for (int i=0;i<chaine.length()-1;i=i+2)
    {
      DD=new DynamicData();
      String comp="";
      comp+=chaine.charAt(i);
      if (chainefinale.indexOf(comp)>0)
      {
        String comp2="";
        comp2+=chaine.charAt(i+1);
        SauverCouleurSommet(Color.magenta,comp);
        SauverCouleurArrete(DD,Color.white,comp,comp2);
        SauverCouleurArrete(DD,Color.white,comp2,comp);
        SauverCouleurSommet(Color.magenta,comp2);
        SauverCouleurArrete(DD,Color.magenta,comp,comp2);
        SauverCouleurArrete(DD,Color.magenta,comp2,comp);
        DD.setCommentaire(algo);
        get_DGD().addStep(DD);
      }
    }
    DD=new DynamicData();
    for (int k=0;k<chainefinale.length()-1;k=k+2)
    {
      DD=new DynamicData();
      String comp="";
      comp+=chainefinale.charAt(k);
      String comp2="";
      comp2+=chainefinale.charAt(k+1);
      SauverCouleurSommet(Color.magenta,comp);
      SauverCouleurSommet(Color.magenta,comp2);
      SauverCouleurArrete(DD,Color.green,comp2,comp);
      SauverCouleurArrete(DD,Color.green,comp,comp2);
      get_DGD().addStep(DD);
    }
    for (int k=0;k<chainefinale.length()-1;k=k+2)
    {
      DD=new DynamicData();
      String comp="";
      comp+=chainefinale.charAt(k);
      String comp2="";
      comp2+=chainefinale.charAt(k+1);
      SauverCouleurArrete(DD,Color.white,comp,comp2);
      SauverCouleurArrete(DD,Color.white,comp2,comp);
      get_DGD().addStep(DD);
    }
  }

  /**
   * Mthode d'amlioration du couplage initial
   * @param s chaine qu'on essaie d'amliorer
   */
  public void Ameliorer(String s)
  {
    DynamicData DD=new DynamicData();
    String algo="";
    String commentaire="";
    commentaire+="\nAmlioration de  la chaine: "+s;
    String n="";
    n += s.charAt(s.length()-1); //on recupere le dernier caractere */
    Vector successeurs=new Vector();
    Sommet l=NOM(n); //rcupration du sommet ayant ce nom
    successeurs=l.getAllSuccesseurs(); //rcup de ses successeurs
    int poi=0;
    String p="";
    boolean teste=true;
    // on regarde si un voisin est non marqu
    while(poi<successeurs.size()&&teste)
    {
      p=((Successeur)(successeurs.elementAt(poi))).getNom();
      String v="";
      int k=0;
      while(!v.equals(p))
      {
        v=((Sommet)Gd.sommet.elementAt(k)).getNom();
        k++;
      }
      k--;
      if(!((Sommet)Gd.sommet.elementAt(k)).getCouple())
        teste=false;
      else poi++;
     }
     boolean fin=false;
     //s'il en existe un,on a une arte non colore
     if(poi<successeurs.size())
     {
       algo+="\n Chaine amliorante: "+s+p;
      DD.setCommentaire(commentaire);
      DD.setAlgo(algo);
      get_DGD().addStep(DD);
      fin=true;
      /**on ajoute le voisin non marqu a la chaine amliorante */
      chainefinale=s+p;
      System.out.println(chainefinale);
      afficher();
      for(int t=0;t<chainefinale.length();t++)
      {
        String x="";
        x+=chainefinale.charAt(t);
        String v="";
        int k=0;
        while(!v.equals(x))
        {
          v=((Sommet)Gd.sommet.elementAt(k)).getNom();
          k++;
        }
        k--;
        ((Sommet)(Gd.sommet.elementAt(k))).setCouple(true);
      }
     }
    //s'il n'y a pas encore de voisin non marqu on continue la recherche
    String temp="";
    if (!fin)
    {
      for(int i=0;i<successeurs.size();i++)
      {
        temp=s;
        String suc=((Successeur)(successeurs.elementAt(i))).getNom();
        if (temp.indexOf(suc)==-1)
        {
          int y=0;
          String cha="";
          cha+=chaine.charAt(y);
          while(!suc.equals(cha))
          {
            y++;
            cha="";
            cha+=chaine.charAt(y);
          }
          String test="";

          if (y%2!=0) test+=chaine.charAt(y-1);
          else test+=chaine.charAt(y+1);
          temp+=suc;
          temp+=test;

          DD=new DynamicData();
          DD.setAlgo(algo);
          DD.setCommentaire(commentaire);
          get_DGD().addStep(DD);
          Ameliorer(temp);
        }
      }
    }
  }

  /** Mthode principale. */
  public void Explorer()
  {
    DynamicData DD=new DynamicData();
    String commentaire="";
    String ori=DYN.getOrigine();
    String v="";
    int i=0;
    //Premire tape :recherche d' un couplage non optimal
    //rcupration du sommet origine
    while(!v.equals(ori))
    {
      v=((Sommet)Gd.sommet.elementAt(i)).getNom();
      i++;
    }
    i--;
    ((Sommet)Gd.sommet.elementAt(i)).setCouple(true);
    //exploration des voisins de l'origine
    Exploration(((Sommet)Gd.sommet.elementAt(i)));
    //exploration de tous les autres sommets
    for(int j=0;j<Taille();j++)
    {
      Sommet c=((Sommet)(Gd.sommet.elementAt(j)));
      if(((Sommet)(Gd.sommet.elementAt(j))).getNom()!=ori && !c.getCouple())
        Exploration((Sommet)(Gd.sommet.elementAt(j)));
    }
    DD.setAlgo("\nPremier couplage trouv: "+chaine);

    get_DGD().addStep(DD);
    //on a trouv un couplage qu'on va chercher  amliorer
    DD = new DynamicData();
    int compteur=0;
    //on compte le nombre de sommets non marqus
    for(int j=0;j<Taille();j++)
      if(!((Sommet)Gd.sommet.elementAt(j)).getCouple())
        compteur++;
     // si 0 ou 1 sommet non marqu : on a le couplage max */
    if (compteur<=1)
    {
      DD=new DynamicData();
      commentaire=("\nFin de L' Algorithme, le couplage obtenu est max");
      DD.setCommentaire(commentaire);
      get_DGD().addStep(DD);
    }
    //sinon on amliore
    //Deuxime tape
    else
    {
      DD=new DynamicData();
      commentaire="\nDEUXIEME ETAPE : recherche de chaines alternes amliorantes ";
      DD.setCommentaire(commentaire);
      get_DGD().addStep(DD);
      //on effectue l' amlioration  partir des sommets non marqus
      for(int k=0;k<Taille();k++)
        if(!((Sommet)Gd.sommet.elementAt(k)).getCouple())
          Ameliorer(((Sommet)Gd.sommet.elementAt(k)).getNom());
    }
    //affichage du couplage max
    DD=new DynamicData();
    commentaire="\nLes artes blanches representent le couplage MAX!!\n";
    DD.setCommentaire(commentaire);
    get_DGD().addStep(DD);
  }

  public void Initialiser()
  {
    for (int i = 0; i < Taille(); i++)
    {
      ((Sommet)Gd.sommet.elementAt(i)).setCouple(false);
      Sommet P = GD().sommetAt(i);
      for (int j=0;j < P.nbrSuccesseur(); j++)
        P.getSuccesseurAt(j).setCouleur(Color.black);
      P.setCouleur(Color.gray);
      P.set_Dist(100000);
      P.set_fermes(false);
    }
    DynamicData DD =new DynamicData();
    DD.setAlgo("\nOrigine: " + DYN.getOrigine());
    DD.setCommentaire("\nINITIALISATION de l'algorithme de Couplage Max\n " );
    Sommet Old = NOM(DYN.getOrigine()).clonePoints();
    NOM(DYN.getOrigine()).setCouleur(Color.blue);
    Sommet New = NOM(DYN.getOrigine()).clonePoints();
    DD.addSommet(Old, New);
    DD.setCommentaire("\n\nPREMIERE ETAPE :Calcul d'un couplage simple");
    get_DGD().addStep(DD);
  }

}
