package grapheanim;

import java.awt.*;
import alxXML.*;

/**
 * Description : Classe permettant de sauver et de recharger des graphes au format XML.
 * Bas sur la version 1.0 des tudiants de maitrise, mais en changeant le package grant le XML.
 * @author <A href=http://www.alapetite.fr.st target=_blank>Alexandre Alapetite</A>
 * @version 2.0
 * @see alxXML
 */
public class FileXML extends Object
{
  private Exception Error = null;
  private AlxXMLdocument alxXMLdocument = null;

  /** Sauvegarde au format XML */
  public boolean SaveToFile(String FileName, DynamicGraphData Data)
  {
    boolean Result = false;
    alxXMLdocument = SaveDynamicGraph(Data);

    //alxXMLdocument.addAttribut("encoding","ISO-8859-1"); //dj par dfaut
    //alxXMLdocument.addAttribut("version","1.0"); //dj par dfaut
    try
    {
      alxXMLdocument.saveToFile(FileName);
      Result = true;
    }
    catch(AlxXMLioException axie)
    {
      //System.out.println(axie.toString()); //dj fait dans les mthodes appelantes
      Error = axie;
      Result = false;
    }
    return Result;
  }

  /** Lecture au format XML */
  public DynamicGraphData ReadFromFile(String FileName)
  {
    DynamicGraphData Result = null;
    alxXMLdocument = new AlxXMLdocument("");
    try
    {
      alxXMLdocument.readFromFile(FileName);
      Result = LoadDynamicGraph(alxXMLdocument.getMainElement());
    }
    catch(AlxXMLioException axie)
    {
      //System.out.println(axie); //dj fait dans les mthodes appelantes
      Error = axie;
      Result = null;
    }
    return Result;
  }

  /** Accesseur sur la dernire erreur d'IO */
  public Exception getLastError()
  {
    return Error;
  }

  // Conversion int,hex,str
  private String IntToStr(int i)
  {
    return String.valueOf(i);
  }

  private int StrToInt(String Str)
  {
    return ((Str!=null)&&(Str.length()>0)) ? Integer.parseInt(Str) : 0;
  }

  private String IntToHex(int i)
  {
    return "0x"+Integer.toHexString(i).substring(2);
  }

  private int HexToInt(String Hex)
  {
    return Integer.decode(Hex).intValue();
  }

  /** IO sur un graphe dynamique (un algo) */
  private AlxXMLdocument SaveDynamicGraph(DynamicGraphData dgd)
  {
    AlxXMLdocument monAlxXMLdocument = new AlxXMLdocument("DYNAMIC_GRAPH");
    AlxXMLelement xdgd = monAlxXMLdocument.getMainElement();
    xdgd.addAttribut("StepSize", IntToStr(dgd.getStepSize()));
    xdgd.addAttribut("Origin", dgd.getOrigine());
    xdgd.addAttribut("Destination", dgd.getArrivee());
    xdgd.addAttribut("Algorithm", dgd.getAlgo());

    xdgd.addXMLelement(SaveGraph(dgd.getInitGraph()));
    dgd.gotoFirstStep();
    for(int i=0;i<dgd.nbrSteps();i++)
    {
      xdgd.addXMLelement(SaveEvol(dgd.getCurrentStep(), i));
      dgd.gotoNextStep();
    }
    xdgd.addAttribut("FinalStep", IntToStr(dgd.getCurrentNumStep()));
    dgd.gotoFirstStep();
    return monAlxXMLdocument;
  }

  private DynamicGraphData LoadDynamicGraph(AlxXMLelement alxXMLelement)
  {
    DynamicGraphData dgd = new DynamicGraphData();
    int fs = StrToInt(alxXMLelement.getAttribut("FinalStep"));
    int ss = StrToInt(alxXMLelement.getAttribut("StepSize"));
    dgd.setOrigine(alxXMLelement.getAttribut("Origin"));
    dgd.setArrivee(alxXMLelement.getAttribut("Destination"));
    dgd.setAlgo(alxXMLelement.getAttribut("Algorithm"));
    GrapheData gd = LoadGraph(alxXMLelement.getElementList().getInFamille(0,"STATIC_GRAPH"));
    dgd.Init(gd, ss, fs);
    int nbSteps = alxXMLelement.getElementList().sizeFamille("STEP");
    for(int i=0;i<nbSteps; i++)
      dgd.addStep(LoadEvol(alxXMLelement.getElementList().getInFamille(i,"STEP")));
    return dgd;
  }

  //IO pour faire voluer un graphe
  private AlxXMLelement SaveEvol(DynamicData dd, int n)
  {
    AlxXMLelement xdd = new AlxXMLelement("STEP");
    xdd.addAttribut("Num", IntToStr(n));
    xdd.addAttribut("Comments", dd.getCommentaire().replace('\n', '@'));
    xdd.addAttribut("Algorithm", dd.getAlgo().replace('\n', '@'));
    AlxXMLelement xold = new AlxXMLelement("OLDS");
    AlxXMLelement xnew = new AlxXMLelement("NEWS");
    for(int i=0;i<dd.getOldSommets().size();i++)
      xold.addXMLelement(SaveSommet((Sommet)dd.getOldSommets().elementAt(i)));
    for(int i=0;i<dd.getNewSommets().size();i++)
      xnew.addXMLelement(SaveSommet((Sommet)dd.getNewSommets().elementAt(i)));
    xdd.addXMLelement(xold);
    xdd.addXMLelement(xnew);
    return xdd;
  }

  private DynamicData LoadEvol(AlxXMLelement e)
  {
    DynamicData dd = new DynamicData();
    dd.setCommentaire(e.getAttribut("Comments").replace( '@', '\n'));
    dd.setAlgo(e.getAttribut("Algorithm").replace( '@', '\n'));
    AlxXMLelement eo = e.getElementList().getInFamille(0,"OLDS");
    AlxXMLelement en = e.getElementList().getInFamille(0,"OLDS");
    int nbNode = eo.getElementList().sizeFamille("NODE");
    for(int i=0;i<nbNode; i++)
    {
      Sommet os = LoadSommet(eo.getElementList().getInFamille(i,"NODE"));
      Sommet ns = LoadSommet(en.getElementList().getInFamille(i,"NODE"));
      dd.addSommet(os, ns);
    }
    return dd;
  }

  //IO sur un graphe statique
  private AlxXMLelement SaveGraph(GrapheData graph)
  {
    AlxXMLelement xgraph = new AlxXMLelement("STATIC_GRAPH");
    for(int i=0;i<graph.nbrSommet();i++)
      xgraph.addXMLelement(SaveSommet(graph.sommetAt(i)));
    return xgraph;
  }

  private GrapheData LoadGraph(AlxXMLelement e)
  {
    GrapheData graph = new GrapheData();
    int nbNode = e.getElementList().sizeFamille("NODE");
    for(int i=0;i<nbNode;i++)
      graph.addSommet(LoadSommet(e.getElementList().getInFamille(i,"NODE")));
    return graph;
  }

  //IO sur les sommets
  private AlxXMLelement SaveSommet(Sommet sommet)
  {
    AlxXMLelement xsommet = new AlxXMLelement("NODE");
    xsommet.addAttribut("Name", sommet.getNom());
    xsommet.addAttribut("X", IntToStr(sommet.getX()));
    xsommet.addAttribut("Y", IntToStr(sommet.getY()));
    xsommet.addAttribut("ColorRGB", IntToHex(sommet.getCouleur().getRGB()));
    xsommet.addAttribut("Weight", IntToStr(sommet.getPoid()));
    xsommet.addAttribut("Flot", IntToStr(sommet.getFlot()));
    xsommet.addAttribut("Heuristic", IntToStr(sommet.get_h()));
    for(int i=0;i<sommet.nbrSuccesseur();i++)
      xsommet.addXMLelement(SaveSucc(sommet.getSuccesseurAt(i)));
    return xsommet;
  }

  private Sommet LoadSommet(AlxXMLelement e)
  {
    Sommet som = new Sommet();
    som.setNom(e.getAttribut("Name"));
    som.setX(StrToInt(e.getAttribut("X")));
    som.setY(StrToInt(e.getAttribut("Y")));
    som.setCouleur(new Color(HexToInt(e.getAttribut("ColorRGB"))));
    som.setPoid(StrToInt(e.getAttribut("Weight")));
    som.setFlot(StrToInt(e.getAttribut("Flot")));
    som.set_h(StrToInt(e.getAttribut("Heuristic")));
    int nbArcs = e.getElementList().sizeFamille("ARCS");
    for(int i=0;i<nbArcs;i++)
      som.addSuccesseur(LoadSucc(e.getElementList().getInFamille(i,"ARCS")));
    return som;
  }

  //IO sur les successeurs (arcs) d'un sommet.
  private AlxXMLelement SaveSucc(Successeur succ)
  {
    AlxXMLelement xsucc = new AlxXMLelement("ARCS");
    xsucc.addAttribut("Destination", succ.getNom());
    xsucc.addAttribut("Weight", IntToStr(succ.getPoid()));
    xsucc.addAttribut("Flot", IntToStr(succ.getFlot()));
    xsucc.addAttribut("ColorRGB", IntToHex(succ.getCouleur().getRGB()));
    return xsucc;
  }

  private Successeur LoadSucc(AlxXMLelement e)
  {
    Successeur succ = new Successeur();
    succ.setNom(e.getAttribut("Destination"));
    succ.setPoid(StrToInt(e.getAttribut("Weight")));
    succ.setFlot(StrToInt(e.getAttribut("Flot")));
    succ.setCouleur(new Color(HexToInt(e.getAttribut("ColorRGB"))));
    return succ;
  }

}
