package grapheanim;

import java.awt.*;
import java.awt.event.*;
import java.util.Properties;

/** Zone d'affichage des graphes dans l'diteur. */
class Graphe extends Component implements ActionListener, WindowListener
{
  private static int decalageSouris = 14;
  private Point dernierClick = new Point();
  private Point pointTemp = new Point();
  private int compteur_lettre = 0;
  private DynamicGraphData stockage = new DynamicGraphData();
  private PopupMenu popup;
  private Frame frame;
  private Image offscreen;
  private PetiteBoite boite;

  public Graphe(Frame frame)
  {
    this.frame = frame;
    stockage.setAlgo("editeur");

    // Les events qui nous interessent
    this.enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    this.enableEvents(AWTEvent.MOUSE_MOTION_EVENT_MASK);
    this.enableEvents(AWTEvent.WINDOW_EVENT_MASK);

    // Creation du popup menu
    popup = new PopupMenu();
    this.add(popup);
  }

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() instanceof MenuItem)
    {
      MenuItem menuItem=(MenuItem)(event.getSource());
      String command = event.getActionCommand();
      if (command.equals("clear"))
      {
        stockage.getInitGraph().removeAllSommets();
        stockage.setAlgo("editeur");
        compteur_lettre = 0;
      }
      else if (command.equals("imprimer")) imprimer(frame);
      else if (command.equals("sauver"))
      {
        stockage.sauver(frame);
        stockage.setAlgo("editeur");
      }
      else if (command.equals("ouvrir"))
      {
        stockage.ouvrir(frame);
        stockage.setAlgo("editeur");
        compteur_lettre = stockage.getInitGraph().nbrSommet();
      }
      else if (command.equals("enlever")) stockage.getInitGraph().enlever(dernierClick);
      else if (command.equals("nommer")) nommer();
      else if (command.equals("heuristique")) heuristique();
      else if (menuItem.getLabel().startsWith("Flot")) changerFlot(command);
      else if (menuItem.getLabel().startsWith("Poid")) changerPoid(command);
      repaint();
    }
  }

  public void paint(Graphics g)
  {
    //if(offscreen == null)
      offscreen = createImage(getSize().width, getSize().height);
    Graphics og = offscreen.getGraphics();
    og.setColor(Color.lightGray);
    og.fillRect(0,0,getSize().width, getSize().height);
    og.setClip(0,0,getSize().width, getSize().height);
    //stockage.afficher(og);
    stockage.afficher(og,false,true);
    //g.fillRect(0,0,getSize().width, getSize().height); //pas ncessaire
    g.drawImage(offscreen, 0, 0, null);
    og.dispose();
  }

  /** vnement de fermeture d'une boite de dialogue PetiteBoite */
  public void windowClosed(WindowEvent evt) { repaint(); }
  public void windowClosing(WindowEvent evt) {}
  public void windowDeiconified(WindowEvent evt) { }
  public void windowIconified(WindowEvent evt) { }
  public void windowActivated(WindowEvent evt) { }
  public void windowDeactivated(WindowEvent evt) { }
  public void windowOpened(WindowEvent evt) { }

  // appelee lorsqu'il y a un MouseEvent
  public void processMouseEvent(MouseEvent e)
  {
    Sommet s = stockage.getInitGraph().getSommet(new Point(e.getX(), e.getY()));
    Sommet origine = stockage.getInitGraph().getSommet(dernierClick);
    if (e.isMetaDown() || e.isPopupTrigger()) //si le bouton droit est enfonc
    {
      dernierClick = new Point(e.getX(), e.getY());
      if (s != null) sommetOptionPopup(true, s);
      else sommetOptionPopup(false, s);
      popup.show(this, e.getX(), e.getY());
    }
    else if (e.getID() == MouseEvent.MOUSE_CLICKED)
    {
      dernierClick = new Point(e.getX()-decalageSouris, e.getY()-decalageSouris);
      if (stockage.getInitGraph().getSommet(dernierClick) == null)
      {
        stockage.getInitGraph().addSommet(new Sommet(dernierClick, getLettre()));
        compteur_lettre++;;
        repaint();
      }
    }
    else if ((e.getID() == MouseEvent.MOUSE_RELEASED))// && (e.getID() != MouseEvent.MOUSE_CLICKED))
    {
      if ((s!=null)&&(origine!=null)&&(s!=origine)&&(origine.getSuccesseur(s.getNom())==null))
        stockage.getInitGraph().addSuccesseur(origine, s);
      repaint();
    }
    else if ((e.getID() == MouseEvent.MOUSE_PRESSED)) dernierClick = new Point(e.getX(), e.getY());
    else super.processMouseEvent(e); // donner a la superclasse pour l'action par defaut
    //dernierClick = new Point(e.getX(), e.getY());
  }

  // appelee losrqu'il y a un MouseMotionEvent
  public void processMouseMotionEvent(MouseEvent e)
  {
    if (e.getID() == MouseEvent.MOUSE_DRAGGED)
    {
      if (stockage.getInitGraph().getSommet(dernierClick) != null)
      {
        Graphics g = getGraphics();
        /*
        g.setColor(Color.lightGray);
        g.fillRect(Math.min(dernierClick.x, pointTemp.x),
             Math.min(dernierClick.y, pointTemp.y),
             Math.max(dernierClick.x-pointTemp.x, pointTemp.x-dernierClick.x),
             Math.max(dernierClick.y-pointTemp.y, pointTemp.y-dernierClick.y));
        g.setColor(Color.black);
        g.clipRect(Math.min(dernierClick.x, e.getX()), Math.min(dernierClick.y, e.getY()),
             Math.max(dernierClick.x-e.getX(), e.getX()-dernierClick.x),
             Math.max(dernierClick.y-e.getY(), e.getY()-dernierClick.y));
        */
        pointTemp = new Point(e.getX(), e.getY());
        paint(g);
        g.drawLine(dernierClick.x, dernierClick.y, e.getX(), e.getY());
      }
    }
    // important ! donner a la superclasse pour l'action par defaut
    else super.processMouseMotionEvent(e);
  }

  private String getLettre() { return Integer.toString(compteur_lettre); }

  private void sommetOptionPopup(boolean b, Sommet s)
  {
    String[] labels = new String[] { "Clear", "Imprimer", "Sauver", "Ouvrir"};
    String[] commandes = new String[] { "clear", "imprimer", "sauver", "ouvrir"};
    popup.removeAll();
    for(int i=0; i<labels.length; i++)
    {
      MenuItem mi = new MenuItem(labels[i]);
      mi.setActionCommand(commandes[i]);
      mi.addActionListener(this);
      popup.add(mi);
    }
    if (b)
    {
      popup.addSeparator();
      labels = new String[2*s.nbrSuccesseur()+3];
      commandes = new String[2*s.nbrSuccesseur()+3];
      labels[0] = "Enlever";
      commandes[0] = "enlever";
      labels[1] = "Nommer";
      commandes[1] = "nommer";
      labels[2] = "Heuristique";
      commandes[2] = "heuristique";
      int i;
      for (i=0; i < s.nbrSuccesseur(); i++)
      {
        labels[i+3] = "Poid -> ".concat(s.getSuccesseurAt(i).getNom());
        commandes[i+3] = s.getSuccesseurAt(i).getNom();
      }
      i+=3;
      for (int j = 0; j < s.nbrSuccesseur(); j++)
      {
        labels[j+i] = "Flot -> ".concat(s.getSuccesseurAt(j).getNom());
        commandes[j+i] = s.getSuccesseurAt(j).getNom();
      }
      for (int k = 0; k < labels.length; k++)
      {
        MenuItem mi = new MenuItem(labels[k]);
        mi.setActionCommand(commandes[k]);
        mi.addActionListener(this);
        popup.add(mi);
      }
    }
  }

  private void nommer()
  {
    boite = new PetiteBoite(frame, stockage.getInitGraph(), dernierClick, "Nommer un sommet", "Nom");
    boite.addWindowListener(this);
  }

  private void changerPoid(String s)
  {
    boite = new PetiteBoite(frame, stockage.getInitGraph(), dernierClick, "Changer le poid", s);
    boite.addWindowListener(this);
  }

  private void changerFlot(String s)
  {
    boite = new PetiteBoite(frame, stockage.getInitGraph(), dernierClick, "Changer le Flot", s);
    boite.addWindowListener(this);
  }

  private void heuristique()
  {
    boite = new PetiteBoite(frame,stockage.getInitGraph(),dernierClick,"Changer l'heuristique","Heuristique");
    boite.addWindowListener(this);
  }

  //private void painture() {repaint();}

  private void imprimer(Frame fr)
  {
    Toolkit toolkit = this.getToolkit();
    //Toolkit toolkit = Toolkit.getDefaultToolkit();
    // "new Properties()" sert a garder les options choisies dans la boite dialogue
    PrintJob job = toolkit.getPrintJob(fr, "Graphe", new Properties());
    if (job == null) return;
    Graphics page = job.getGraphics();
    Dimension taille = this.getSize();
    //Dimension taille = this.getScreenSize();
    Dimension taillePage = job.getPageDimension();
    // centrer la page sur la feuille
    page.translate((taillePage.width - taille.width)/2,
             (taillePage.height - taille.height)/2);
    page.drawRect(0, 0, taille.width, taille.height);
    // necessaire pour que le graphe ne sorte pas de la feuille...
    page.setClip(0, 0, taille.width, taille.height);
    this.print(page);
    page.dispose();
    job.end();
  }

}
