package grapheanim;

import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.font.*;
import java.io.*;
import java.util.*;
import java.util.Properties;

/** Classe grant un graphe avec sommets, mthodes d'affichage. */
public class GrapheData implements Serializable
{
  protected Vector sommet;
  /**
   * Table de hachage contenant aussi les sommets. Mmes pointeurs que dans le <CODE>Vector</CODE> sommet.
   * @see #sommet
   */
  protected Hashtable htSommets;
  /** indicateur prcisant si on veut que soit affich le poids et le flot */
  private byte poids_flot;
  private static final byte RIEN=0;
  private static final byte POIDS=1;
  private static final byte POIDS_FLOT=2;
  private byte poids_sommet;
  /** indicateur prcisant si on veut que soit affich l'heuristique */
  private boolean heuristique;

  //les constructeurs
  GrapheData()
  {
    sommet = new Vector();
    htSommets = new Hashtable();
  }

  GrapheData(GrapheData gd)
  {
    this();
    for(int i=0;i<gd.nbrSommet();i++)
      addSommet(gd.sommetAt(i).clonePoints());
      //sommet.addElement(gd.sommetAt(i).clonePoints());
  }

  //les accesseurs
  public int nbrSommet() { return sommet.size(); }
  public Sommet sommetAt(int i) { return (Sommet)sommet.elementAt(i); }
  public void addSommet(Sommet s)
  {
    sommet.add(s);
    htSommets.put(s.getNom(),s);
  }
  public void removeSommet(Sommet s)
  {
    htSommets.remove(s.getNom());
    sommet.removeElement(s);
  }
  public void removeSommetAt(int i)
  {
    Sommet aSommet = (Sommet)sommet.remove(i);
    htSommets.remove(aSommet.getNom());
  }
  public void setSommetAt(Sommet s, int i)
  {
    //sommet.setElementAt(s,i);
    htSommets.remove(((Sommet)sommet.elementAt(i)).getNom());
    htSommets.put(s.getNom(),s);
    sommet.setElementAt(s,i);
  }
  public int indexOf(Sommet s) { return sommet.indexOf(s); }
  public int indexOf(String st)
  {
    /*boolean trouve = false;
    int cpt = 0;
    while (!trouve && cpt<sommet.size())
    {
      trouve = ((Sommet)sommet.elementAt(cpt)).getNom().equals(st);
      cpt++;
    }
    return trouve ? --cpt : -1;*/
    Sommet aSommet = (Sommet)htSommets.get(st);
    return sommet.indexOf(aSommet);
  }
  public Sommet getSommet(String st)
  {
    /*for (int i = 0; i < sommet.size(); i++)
    {
      Sommet s = (Sommet)sommet.elementAt(i);
      if (s.getNom().equals(st)) return s;
    }
    return null;*/
    return (Sommet)htSommets.get(st);
  }
  public void removeArete(Sommet s)
  {
    for (int i = 0; i < sommet.size(); i++)
    {
      Sommet som = (Sommet)sommet.elementAt(i);
      if (som.getSuccesseur(s.getNom()) != null) som.removeSuccesseur(s.getNom());
    }
  }
  public Vector getAllSommets() { return sommet; }
  public void removeAllSommets()
  {
    sommet.clear(); //removeAllElements();
    htSommets.clear();
  }

  /** @return le sommet existant  cette position, null sinon */
  public Sommet getSommet(Point p)
  {
    Rectangle r = new Rectangle();
    Sommet s;
    for (int i = 0; i < sommet.size(); i++)
    {
      s = (Sommet)sommet.elementAt(i);
      //Rectangle r = new Rectangle(s.getLocation().x-13,s.getLocation().y-13, 50,50);
      r.setBounds(s.getLocation().x-13,s.getLocation().y-13, 50,50);
      if (r.contains(p)) return s;
    }
    return null;
  }

  public void renameSommet(Sommet s, String st)
  {
    htSommets.remove(s.getNom());
    for (int i = 0; i < sommet.size(); i++)
    {
      Sommet som = (Sommet)sommet.elementAt(i);
      if (som.getSuccesseur(s.getNom()) != null)
        som.renameSuccesseur(s.getNom(), st);
    }
    s.setNom(st);
    htSommets.put(s.getNom(),s);
  }

  public void addSuccesseur(Sommet d, Sommet a)
  {
    int i = (int) (Math.sqrt((a.getX()-d.getX())*(a.getX()-d.getX()) +
      (a.getY()-d.getY())*(a.getY()-d.getY()) ));
    i = (int) ((i*100)/800);
    d.addSuccesseur(a.getNom(), i, 0);
  }

  /**Cherche le successeur matrialisant l'arc d-a, le cre s'il n'existe pas, et change son poid  i */
  public void changePoidSuccesseur(Sommet d, Sommet a, int i)
  {
    if (i == 0) d.removeSuccesseur(a.getNom());
    else
    {
      Successeur successeur = d.getSuccesseur(a.getNom());
      if (successeur==null) d.addSuccesseur(a.getNom(),i);
      else successeur.setPoid(i);
    }
  }

  /** Cherche le successeur matrialisant l'arc d-a, le cre s'il n'existe pas, et change son flot  i */
  public void changeFlotSuccesseur(Sommet d, Sommet a, int i)
  {
    Successeur successeur = d.getSuccesseur(a.getNom());
    if (successeur==null) d.addSuccesseur(a.getNom(),Sommet.DIST_MAX);
    else successeur.setFlot(i);
  }

  public void afficher(Graphics g, String str)
  {
    if (str.equals("editeur") || str.equals("Flot")) poids_flot=POIDS_FLOT;
    else if (str.equals("Largeur") || str.equals("Lexico") || str.equals("Coloration") || str.equals("Biparti")) poids_flot=RIEN;
    else poids_flot=POIDS;

    if (str.equals("Coloration")) poids_sommet=POIDS;
    else poids_sommet = RIEN;
    heuristique = str.equals("A*") || str.equals("A") || str.equals("editeur");// || str.equals("Lexico");

    g.setFont(new Font("Helvetica",Font.PLAIN,11));
    for(int i = 0; i < nbrSommet(); i++) afficherArc(sommetAt(i), g, str);
    for(int i = 0; i < nbrSommet(); i++) afficherSommet(sommetAt(i), g, str);
  }

  public void afficherArc(Sommet s, Graphics g, String str)
  {
    for(int j = 0; j < s.nbrSuccesseur(); j++)
    {
      Sommet dest = getSommet(s.getSuccesseurAt(j).getNom());
      if (dest != null)
      {
        Point p = dest.getLocation();
        Point p1 = new Point(s.getLocation().x+12, s.getLocation().y+12);
        Point p2 = new Point(p.x+12, p.y+12);
        Complex c = new Complex();
        c.setFromRectangulaire(p1, p2);

        Successeur suc1 = s.getSuccesseurAt(j);
        Successeur suc2 = getSommet(p).getSuccesseur(s.getNom());
        if (suc2 != null)
        {
          //pour un aller-retour...
          double alpha = c.alpha;
          double longueur = c.longueur;
          c.setFromPolaire(alpha+(Math.PI/6), 12.5);
          p1.translate((int)c.X, (int)c.Y);
          c.setFromPolaire(alpha, longueur-22);
          p2.setLocation(p1.x+(int)c.X, p1.y+(int)c.Y);
          g.setColor(s.getSuccesseurAt(j).getCouleur());
          if((str.equals("Biparti"))||(str.equals("Coloration")))
          {
             p1 = new Point(s.getLocation().x+12, s.getLocation().y+12);
             p2 = new Point(p.x+12, p.y+12);
             g.drawLine(p1.x, p1.y, p2.x ,p2.y);
          }
          else g.drawLine(p1.x, p1.y, p2.x ,p2.y);
          c.setFromPolaire(alpha, 17);
          p1.translate((int)c.X, (int)c.Y);
          //afficherFleche(p1, p2, suc1.getPoid(),suc1.getFlot(), g, str);
          if((!str.equals("Biparti"))&&(!str.equals("Coloration")))
            afficherFleche(p1, p2, suc1.getPoid(),suc1.getFlot(), g);
        }
        else
        {
          c.setFromPolaire(c.alpha, 12.5);
          p1.setLocation((int)(p1.x+c.X), (int)(p1.y+c.Y));
          p2.setLocation((int)(p2.x-c.X), (int)(p2.y-c.Y));

          g.setColor(s.getSuccesseurAt(j).getCouleur());
          g.drawLine(p1.x, p1.y, p2.x, p2.y);
          p1.translate((int)c.X*2, (int)c.Y*2);
          //afficherFleche(p1, p2, suc1.getPoid(),suc1.getFlot(), g, str);
          if ((!str.equals("Biparti"))&&(!str.equals("Coloration")))
            afficherFleche(p1, p2, suc1.getPoid(),suc1.getFlot(), g);
        }
      }
      //afficherSommet(dest, g, str);
    }
  }

  public void afficherTexte(String texte, int x, int y, Graphics g)
  {
    FontMetrics fontMetrix = g.getFontMetrics(g.getFont());
    Color couleur = g.getColor();
    g.setColor(Color.lightGray);
    g.fillRect(x,y-(1+fontMetrix.getHeight()/2),fontMetrix.stringWidth(texte),(3+fontMetrix.getHeight()/2)); //nettoyage de la zone pour crire
    g.setColor(couleur);
    g.drawString(texte,x,y);
  }

  public void afficherFleche(Point d, Point a, int p, int f, Graphics g)
  {
    //affichage de la fleche
    int [] x = new int[3];
    int [] y = new int[3];
    //if (str.equals("Largeur") || (str.equals("Lexico")) || (str.equals("Coloration"))) { p = 50; }
    if (poids_flot==RIEN) p = 50; //si le poids n'est pas important pour l'algo, on met la flche au milieu de l'arc
    x[0] = d.x + (a.x - d.x) * p / 100;
    y[0] = d.y + (a.y - d.y) * p  / 100;
    Complex c = new Complex();
    c.setFromRectangulaire(a, d);
    double alpha = c.alpha;
    double longueur = c.longueur;
    c.setFromPolaire(alpha+(Math.PI/8), 15);
    x[1] = (int) (x[0]+c.X);
    y[1] = (int) (y[0]+c.Y);
    c.setFromPolaire(alpha-(Math.PI/8), 15);
    x[2] = (int) (x[0]+c.X);
    y[2] = (int) (y[0]+c.Y);
    g.fillPolygon(x,y,3);
    //affichage du poid et du flot
    if (poids_flot==POIDS)
    {
      int absi = (int) (x[0]+Math.cos(alpha - (Math.PI/3))*15);
      int ordo = (int) (y[0]+Math.sin(alpha - (Math.PI/3))*15);
      //g.setFont(new Font("Helvetica",Font.PLAIN,11));//Font("t",1,11));
      afficherTexte(String.valueOf(p),absi,ordo,g);
    }
    else if (poids_flot==POIDS_FLOT)
    {
      int absi = (int) (x[0]+Math.cos(alpha - (Math.PI/3))*15);
      int ordo = (int) (y[0]+Math.sin(alpha - (Math.PI/3))*15);
      //g.setFont(new Font("Helvetica",Font.PLAIN,11));//Font("t",1,11));
      afficherTexte(String.valueOf(f)+"/"+String.valueOf(p),absi,ordo,g);
    }
  }

  // /** @deprecated */
  /*
  public void afficherFleche(Point d, Point a, int p, int f, Graphics g, String str)
  {
    //affichage de la fleche
    int [] x = new int[3];
    int [] y = new int[3];
    if (str.equals("Largeur") || (str.equals("Lexico")) || (str.equals("Coloration"))) { p = 50; }
    x[0] = d.x + (a.x - d.x) * p / 100;
    y[0] = d.y + (a.y - d.y) * p  / 100;
    Complex c = new Complex();
    c.setFromRectangulaire(a, d);
    double alpha = c.alpha;
    double longueur = c.longueur;
    c.setFromPolaire(alpha+(Math.PI/8), 15);
    x[1] = (int) (x[0]+c.X);
    y[1] = (int) (y[0]+c.Y);
    c.setFromPolaire(alpha-(Math.PI/8), 15);
    x[2] = (int) (x[0]+c.X);
    y[2] = (int) (y[0]+c.Y);
    g.fillPolygon(x,y,3);
    //Point temp = new Point();
    //affichage du poid
    int absi = (int) (x[0]+Math.cos(alpha - (Math.PI/3))*15);
    int ordo = (int) (y[0]+Math.sin(alpha - (Math.PI/3))*15);
    g.setFont(new Font("Helvetica",Font.PLAIN,11));//Font("t",1,11));
    if (str.equals("editeur")) afficherTexte(String.valueOf(f)+"/"+String.valueOf(p),absi,ordo,g);
    else if (!str.equals("Largeur") && !str.equals("Lexico")&& !str.equals("Coloration"))
      if (str.equals("Flot")) afficherTexte(String.valueOf(f)+"/"+String.valueOf(p),absi,ordo,g);
      else afficherTexte(String.valueOf(p),absi,ordo,g);
  }*/

  public Color negatif(Color couleur)
  {
    //int rgb = couleur.getRGB();
    int r = couleur.getRed();
    int v = couleur.getGreen();
    int b = couleur.getBlue();
    if (r>=(v+b)) return Color.cyan;
    else if (v>=(r+b)) return Color.magenta;
    else if (b>=(r+v)) return Color.yellow;
    else if ((r+v+b)<0x210) return Color.white;
    else if ((r+v+b)>0x17E) return Color.black;
    else return new Color(~couleur.getRGB()|0xFF000000);
  }

  public void afficherSommet(Sommet s, Graphics g, String str)
  {
    //affichage du rond
    g.setColor(s.getCouleur());
    g.fillOval(s.getLocation().x, s.getLocation().y, 25, 25);
    g.setColor(Color.black);
    g.drawOval(s.getLocation().x, s.getLocation().y, 25, 25);
    //affichage du nom de sommet
    g.setFont(new Font("Helvetica",Font.BOLD,12)); //Font("t",1,12));
    //g.setXORMode(Color.white);
    g.setColor(negatif(s.getCouleur()));
    g.drawString(s.getNom(),s.getLocation().x+7, s.getLocation().y+10);
    //g.setPaintMode();
    g.setFont(new Font("Helvetica",Font.PLAIN,8));
    //affichage de l'heuristique
    if (heuristique)
    {
      g.setColor(Color.black);
      g.drawString("[ "+String.valueOf(s.getHeuristique())+" ]",s.getLocation().x+2,s.getLocation().y+19);
    }
    //affichage du poids
    g.setColor(Color.blue);
    if (str.equals("Lexico"))
      g.drawString(s.getDistLexico(), s.getLocation().x+4, s.getLocation().y+17);
    else if (poids_sommet==POIDS && (s.getPoid() < 100000))
    {
      g.setFont(new Font("Helvetica",Font.PLAIN,10));
      if (s.getPoid() > 99)
        afficherTexte(Integer.toString(s.getPoid()),s.getLocation().x+4-10,s.getLocation().y-2,g);
      else if (s.getPoid() > 9)
        afficherTexte(Integer.toString(s.getPoid()),s.getLocation().x+7-10,s.getLocation().y-2,g);
      else afficherTexte(Integer.toString(s.getPoid()),s.getLocation().x+10-10,s.getLocation().y-2,g);
    }
  }

  public void enlever(Point p)
  {
    Sommet s = getSommet(p);
    removeSommet(s);
    removeArete(s);
  }

}
