package graphelastic;

import java.util.StringTokenizer;
import java.awt.*;
import java.awt.event.*;

/**
 * Fentre graphique hbergeant un <CODE>GraphPanel</CODE>, quelques boutons et
 * les vnements souris.
 */
public class GraphAnimFrame extends Frame implements ActionListener, ItemListener
{
  GraphPanel panel;
  Panel controlPanel;

  Button scramble = new Button("Ecartelle");
  Button shake = new Button("Agite");
  Checkbox auto = new Checkbox("Auto");

  public GraphAnimFrame()
  {
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    this.setSize(750, 550);
    this.setTitle("Graphe color lastique");
    setLayout(new BorderLayout());

    panel = new GraphPanel(this);
    add("Center", panel);
    controlPanel = new Panel();
    add("South", controlPanel);

    controlPanel.add(scramble);
    scramble.addActionListener(this);
    controlPanel.add(shake);
    shake.addActionListener(this);
    auto.setState(true);
    controlPanel.add(auto);
    auto.addItemListener(this);
  }

  public void destroy()
  {
    panel.destroy();
    remove(panel);
    remove(controlPanel);
    dispose();
  }

  public void start()
  {
    panel.start();
  }

  public void setAuto(boolean aut)
  {
    this.auto.setState(aut);
    this.panel.auto = aut;
  }

  public void addCenter(SommetInterface sommet)
  {
    Node n = panel.findNode(sommet);
    n.x = getWidth() / 2;
    n.y = getHeight() / 2;
    n.fixed = true;
  }

  public void addNode(SommetInterface sommet, double x, double y)
  {
    panel.addNode(sommet,x,y);
  }

  public void addEdge(SommetInterface pere, SommetInterface fils)
  {
    addEdge(pere, fils, 100);
  }

  public void addEdge(SommetInterface pere, SommetInterface fils, int longueurArc)
  {
    panel.addEdge(pere, fils, longueurArc);
  }

  public void actionPerformed(ActionEvent e)
  {
    Object src = e.getSource();
    if (src == scramble)
    {
      Dimension d = getSize();
      for (int i = 0 ; i < panel.nnodes ; i++)
      {
        Node n = panel.nodes[i];
        if (!n.fixed)
        {
          n.x = 10 + (d.width-20)*Math.random();
          n.y = 10 + (d.height-20)*Math.random();
        }
      }
    }
    else if (src == shake)
    {
      Dimension d = getSize();
      for (int i = 0 ; i < panel.nnodes ; i++)
      {
        Node n = panel.nodes[i];
        if (!n.fixed)
        {
          n.x += 80*Math.random() - 40;
          n.y += 80*Math.random() - 40;
        }
      }
    }
  }

  public void itemStateChanged(ItemEvent e)
  {
    Object src = e.getSource();
    boolean on = e.getStateChange() == ItemEvent.SELECTED;
    if (src == auto) panel.auto = on;
  }

  protected void processWindowEvent(WindowEvent e)
  {
    super.processWindowEvent(e);
    if (e.getID() == WindowEvent.WINDOW_CLOSING) destroy();
  }

}
