package graphelastic;

import java.awt.*;
import java.awt.event.*;

/**
 * Affiche les sommets, et essaye de satisfaire les contraintes de toutes
 * les arrtes.
 */
class GraphPanel extends Panel implements Runnable, MouseListener, MouseMotionListener
{
  private GraphAnimFrame graph;
  int nnodes;
  Node nodes[] = new Node[200];

  private int nedges;
  private Edge edges[] = new Edge[400];

  boolean auto;

  private Thread relaxer;

  public GraphPanel(GraphAnimFrame graph)
  {
    this.graph = graph;
    addMouseListener(this);
    auto = true;
  }

  public void destroy()
  {
    stop();
    edges = null;
    nodes = null;
    pick = null;
    if (offgraphics != null) offgraphics.dispose();
    offgraphics = null;
    offscreen = null;
    offscreensize = null;
  }

  Node findNode(SommetInterface sommet)
  {
    int id = sommet.getIdentifiant();
    for (int i = 0 ; i < nnodes ; i++)
      if (id==nodes[i].sommet.getIdentifiant()) return nodes[i];
    return addNode(sommet,10 + 380*Math.random(),10 + 380*Math.random());
  }

  Node addNode(SommetInterface sommet, double x, double y)
  {
    Node n = new Node();
    n.x = x;
    n.y = y;
    n.sommet = sommet;
    nodes[nnodes] = n;
    nnodes++;
    return n;
  }

  void addEdge(SommetInterface from, SommetInterface to, int len)
  {
    Edge e = new Edge();
    e.nodeFrom = findNode(from);
    e.nodeTo = findNode(to);
    e.len = len;
    edges[nedges++] = e;
  }

  public void run()
  {
    Thread me = Thread.currentThread();
    while (relaxer == me)
    {
      if (auto) relax();
      else me.yield();
      repaint();
      try
      {
        Thread.sleep(150);
      }
      catch (InterruptedException e)
      {
        break;
      }
      me.yield();
    }
  }

  synchronized void relax()
  {
    for (int i = 0 ; i < nedges ; i++) edges[i].relax();
    for (int i = 0 ; i < nnodes ; i++)
    {
      Node n1 = nodes[i];
      double dx = 0;
      double dy = 0;
      for (int j = 0 ; j < nnodes ; j++)
      {
        if (i == j) continue;
        Node n2 = nodes[j];
        double vx = n1.x - n2.x;
        double vy = n1.y - n2.y;
        double len = vx * vx + vy * vy;
        if (len == 0)
        {
          dx += Math.random();
          dy += Math.random();
        }
        else if (len < 100*100)
        {
          dx += vx / len;
          dy += vy / len;
        }
      }
      double dlen = dx * dx + dy * dy;
      if (dlen > 0)
      {
        dlen = Math.sqrt(dlen) / 2;
        n1.dx += dx / dlen;
        n1.dy += dy / dlen;
      }
    }
    Dimension d = getSize();
    for (int i = 0 ; i < nnodes ; i++) nodes[i].positionne(d);
  }

  private Node pick;
  private boolean pickfixed;
  private Image offscreen;
  private Dimension offscreensize;
  private Graphics offgraphics;

  public synchronized void update(Graphics g)
  {
    Dimension d = getSize();
    if ((offscreen == null) || (d.width != offscreensize.width) || (d.height != offscreensize.height))
    {
        offscreen = createImage(d.width, d.height);
        offscreensize = d;
        if (offgraphics != null) offgraphics.dispose();
        offgraphics = offscreen.getGraphics();
        offgraphics.setFont(getFont());
    }
    offgraphics.setColor(getBackground());
    offgraphics.fillRect(0, 0, d.width, d.height);
    offgraphics.setColor(Color.black);
    for (int i = 0 ; i < nedges ; i++) edges[i].paint(offgraphics);
    FontMetrics fm = offgraphics.getFontMetrics();
    for (int i = 0 ; i < nnodes ; i++) nodes[i].paint(offgraphics,fm);
    g.drawImage(offscreen, 0, 0, null);
  }

  /** 1.1 event handling */
  public void mouseClicked(MouseEvent e) { }
  public void mouseEntered(MouseEvent e) { }
  public void mouseExited(MouseEvent e) { }
  public void mouseMoved(MouseEvent e) { }

  public void mousePressed(MouseEvent e)
  {
    addMouseMotionListener(this);
    double bestdist = Double.MAX_VALUE;
    int x = e.getX();
    int y = e.getY();
    for (int i = 0 ; i < nnodes ; i++)
    {
      Node n = nodes[i];
      double dist = (n.x - x) * (n.x - x) + (n.y - y) * (n.y - y);
      if (dist < bestdist)
      {
        pick = n;
        bestdist = dist;
      }
    }
    pickfixed = pick.fixed;
    pick.fixed = true;
    pick.x = x;
    pick.y = y;
    repaint();
    e.consume();
  }

  public void mouseReleased(MouseEvent e)
  {
    removeMouseMotionListener(this);
    if (pick != null)
    {
      pick.x = e.getX();
      pick.y = e.getY();
      pick.fixed = pickfixed;
      pick = null;
    }
    repaint();
    e.consume();
  }

  public void mouseDragged(MouseEvent e)
  {
    pick.x = e.getX();
    pick.y = e.getY();
    repaint();
    e.consume();
  }

  public void start()
  {
    relaxer = new Thread(this);
    relaxer.start();
  }

  public void stop()
  {
    relaxer = null;
  }

}
