package graphelastic;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.FontMetrics;

/**
 * Reprsentation graphique d'un sommet.
 * @see algod.Sommet
 */
class Node extends Object
{
  /**
   * Tableau des codes hexadcimaux des couleurs  utiliser.
   */
  private static String[] couleurs =
  {
    "0xFFFFFF", //blanc
    "0x00FF00", //vert
    "0xFFFF00", //jaune
    "0xFF7F00", //orange
    "0x00FFFF", //cyan
    "0xBF00FF", //violet
    "0x0000FF", //bleu
    "0xFF00FF", //magenta
    "0xFF007F", //rose
    "0x777777", //gris
  };

  private final static int marge = 10;

  /**
   * Abscisse du sommet.
   */
  double x;

  /**
   * Ordonne du point.
   */
  double y;

  double dx;
  double dy;

  /**
   * Indique si ce sommet peut bouger pour respecter les contraintes des arcs, ou pas.
   */
  boolean fixed;

  /**
   * Sommet rel. Utilis pour connatre les modifications de couleur.
   */
  SommetInterface sommet;

  /**
   * Accs  la couleur de ce sommet.
   */
  public final Color getCouleur()
  {
    return Color.decode(couleurs[sommet.getCouleur() % couleurs.length]);
  }

  /**
   * Accs  la couleur du bord du sommet (rouge si lu, noir sinon).
   */
  public final Color getBorderCouleur()
  {
    return sommet.isElu() ? Color.red  : Color.black;
  }

  public void positionne(Dimension d)
  {
    if (!fixed)
    {
      x += Math.max(-5, Math.min(5, dx));
      y += Math.max(-5, Math.min(5, dy));
    }
    if (x < marge) x = marge;
    else if (x > d.width-marge) x = d.width-marge;
    if (y < marge) y = marge;
    else if (y > d.height-marge) y = d.height-marge;
    dx /= 2;
    dy /= 2;
  }

  public void paint(Graphics g, FontMetrics fm)
  {
    int ix = (int)x;
    int iy = (int)y;
    String lbl = sommet.getLabel();
    int w = fm.stringWidth(lbl) + 10;
    int h = fm.getHeight() + 4;
    g.setColor(getCouleur());
    g.fillRect(ix - w/2, iy - h /2, w, h);
    g.setColor(Color.black);
    g.drawString(lbl, ix - (w-10)/2, (iy - (h-4)/2) + fm.getAscent());
    if (sommet.isElu())
    {
      g.drawRect(ix - w/2, iy - h/2, w-1, h-1);
      g.setColor(Color.red);
      g.drawRect(ix - 2 - w/2, iy - 2 - h/2, w+3, h+3);
      g.drawRect(ix - 1 - w/2, iy - 1 - h/2, w+1, h+1);
    }
    else if (sommet.isCouleurOfficielle())
    {
      //g.setColor(Color.black);
      g.drawRect(ix - w/2, iy - h/2, w-1, h-1);
      g.drawRect(ix - 1 - w/2, iy - 1 - h/2, w+1, h+1);
    }
  }

}
